# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .ad_hoc_execute_prompt_event import AdHocExecutePromptEvent
from .ad_hoc_expand_meta import AdHocExpandMeta
from .ad_hoc_fulfilled_prompt_execution_meta import AdHocFulfilledPromptExecutionMeta
from .ad_hoc_initiated_prompt_execution_meta import AdHocInitiatedPromptExecutionMeta
from .ad_hoc_rejected_prompt_execution_meta import AdHocRejectedPromptExecutionMeta
from .ad_hoc_streaming_prompt_execution_meta import AdHocStreamingPromptExecutionMeta
from .add_openai_api_key_enum import AddOpenaiApiKeyEnum
from .api_node_result import ApiNodeResult
from .api_node_result_data import ApiNodeResultData
from .api_request_parent_context import ApiRequestParentContext
from .api_version_enum import ApiVersionEnum
from .array_chat_message_content import ArrayChatMessageContent
from .array_chat_message_content_item import ArrayChatMessageContentItem
from .array_chat_message_content_item_request import ArrayChatMessageContentItemRequest
from .array_chat_message_content_request import ArrayChatMessageContentRequest
from .array_input import ArrayInput
from .array_vellum_value import ArrayVellumValue
from .array_vellum_value_request import ArrayVellumValueRequest
from .audio_chat_message_content import AudioChatMessageContent
from .audio_chat_message_content_request import AudioChatMessageContentRequest
from .audio_prompt_block import AudioPromptBlock
from .audio_vellum_value import AudioVellumValue
from .audio_vellum_value_request import AudioVellumValueRequest
from .base_output import BaseOutput
from .basic_vectorizer_intfloat_multilingual_e_5_large import BasicVectorizerIntfloatMultilingualE5Large
from .basic_vectorizer_intfloat_multilingual_e_5_large_request import BasicVectorizerIntfloatMultilingualE5LargeRequest
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_cos_v_1 import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1,
)
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_cos_v_1_request import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request,
)
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_dot_v_1 import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1,
)
from .basic_vectorizer_sentence_transformers_multi_qa_mpnet_base_dot_v_1_request import (
    BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request,
)
from .build_status_enum import BuildStatusEnum
from .chat_history_input import ChatHistoryInput
from .chat_history_input_request import ChatHistoryInputRequest
from .chat_history_vellum_value import ChatHistoryVellumValue
from .chat_history_vellum_value_request import ChatHistoryVellumValueRequest
from .chat_message import ChatMessage
from .chat_message_content import ChatMessageContent
from .chat_message_content_request import ChatMessageContentRequest
from .chat_message_prompt_block import ChatMessagePromptBlock
from .chat_message_request import ChatMessageRequest
from .chat_message_role import ChatMessageRole
from .code_execution_node_array_result import CodeExecutionNodeArrayResult
from .code_execution_node_chat_history_result import CodeExecutionNodeChatHistoryResult
from .code_execution_node_error_result import CodeExecutionNodeErrorResult
from .code_execution_node_function_call_result import CodeExecutionNodeFunctionCallResult
from .code_execution_node_json_result import CodeExecutionNodeJsonResult
from .code_execution_node_number_result import CodeExecutionNodeNumberResult
from .code_execution_node_result import CodeExecutionNodeResult
from .code_execution_node_result_data import CodeExecutionNodeResultData
from .code_execution_node_result_output import CodeExecutionNodeResultOutput
from .code_execution_node_search_results_result import CodeExecutionNodeSearchResultsResult
from .code_execution_node_string_result import CodeExecutionNodeStringResult
from .code_execution_package import CodeExecutionPackage
from .code_execution_runtime import CodeExecutionRuntime
from .code_executor_input import CodeExecutorInput
from .code_executor_response import CodeExecutorResponse
from .code_executor_secret_input import CodeExecutorSecretInput
from .code_resource_definition import CodeResourceDefinition
from .compile_prompt_deployment_expand_meta_request import CompilePromptDeploymentExpandMetaRequest
from .compile_prompt_meta import CompilePromptMeta
from .components_schemas_pdf_search_result_meta_source import ComponentsSchemasPdfSearchResultMetaSource
from .components_schemas_pdf_search_result_meta_source_request import ComponentsSchemasPdfSearchResultMetaSourceRequest
from .components_schemas_prompt_version_build_config_sandbox import ComponentsSchemasPromptVersionBuildConfigSandbox
from .condition_combinator import ConditionCombinator
from .conditional_node_result import ConditionalNodeResult
from .conditional_node_result_data import ConditionalNodeResultData
from .container_image_build_config import ContainerImageBuildConfig
from .container_image_container_image_tag import ContainerImageContainerImageTag
from .container_image_read import ContainerImageRead
from .create_test_suite_test_case_request import CreateTestSuiteTestCaseRequest
from .deployment_history_item import DeploymentHistoryItem
from .deployment_provider_payload_response import DeploymentProviderPayloadResponse
from .deployment_provider_payload_response_payload import DeploymentProviderPayloadResponsePayload
from .deployment_read import DeploymentRead
from .deployment_release_tag_deployment_history_item import DeploymentReleaseTagDeploymentHistoryItem
from .deployment_release_tag_read import DeploymentReleaseTagRead
from .docker_service_token import DockerServiceToken
from .document_chat_message_content import DocumentChatMessageContent
from .document_chat_message_content_request import DocumentChatMessageContentRequest
from .document_document_to_document_index import DocumentDocumentToDocumentIndex
from .document_index_chunking import DocumentIndexChunking
from .document_index_chunking_request import DocumentIndexChunkingRequest
from .document_index_indexing_config import DocumentIndexIndexingConfig
from .document_index_indexing_config_request import DocumentIndexIndexingConfigRequest
from .document_index_read import DocumentIndexRead
from .document_processing_state import DocumentProcessingState
from .document_prompt_block import DocumentPromptBlock
from .document_read import DocumentRead
from .document_status import DocumentStatus
from .document_vellum_value import DocumentVellumValue
from .document_vellum_value_request import DocumentVellumValueRequest
from .enriched_normalized_completion import EnrichedNormalizedCompletion
from .entity_status import EntityStatus
from .entity_visibility import EntityVisibility
from .environment_enum import EnvironmentEnum
from .ephemeral_prompt_cache_config import EphemeralPromptCacheConfig
from .ephemeral_prompt_cache_config_type_enum import EphemeralPromptCacheConfigTypeEnum
from .error_detail_response import ErrorDetailResponse
from .error_input import ErrorInput
from .error_vellum_value import ErrorVellumValue
from .error_vellum_value_request import ErrorVellumValueRequest
from .event_create_response import EventCreateResponse
from .execute_api_request_bearer_token import ExecuteApiRequestBearerToken
from .execute_api_request_body import ExecuteApiRequestBody
from .execute_api_request_headers_value import ExecuteApiRequestHeadersValue
from .execute_api_response import ExecuteApiResponse
from .execute_api_response_json import ExecuteApiResponseJson
from .execute_prompt_event import ExecutePromptEvent
from .execute_prompt_response import ExecutePromptResponse
from .execute_workflow_response import ExecuteWorkflowResponse
from .execute_workflow_workflow_result_event import ExecuteWorkflowWorkflowResultEvent
from .execution_array_vellum_value import ExecutionArrayVellumValue
from .execution_chat_history_vellum_value import ExecutionChatHistoryVellumValue
from .execution_error_vellum_value import ExecutionErrorVellumValue
from .execution_function_call_vellum_value import ExecutionFunctionCallVellumValue
from .execution_json_vellum_value import ExecutionJsonVellumValue
from .execution_number_vellum_value import ExecutionNumberVellumValue
from .execution_search_results_vellum_value import ExecutionSearchResultsVellumValue
from .execution_string_vellum_value import ExecutionStringVellumValue
from .execution_thinking_vellum_value import ExecutionThinkingVellumValue
from .execution_vellum_value import ExecutionVellumValue
from .external_input_descriptor import ExternalInputDescriptor
from .external_parent_context import ExternalParentContext
from .external_test_case_execution import ExternalTestCaseExecution
from .external_test_case_execution_request import ExternalTestCaseExecutionRequest
from .fast_embed_vectorizer_baai_bge_small_en_v_15 import FastEmbedVectorizerBaaiBgeSmallEnV15
from .fast_embed_vectorizer_baai_bge_small_en_v_15_request import FastEmbedVectorizerBaaiBgeSmallEnV15Request
from .finish_reason_enum import FinishReasonEnum
from .folder_entity import FolderEntity
from .folder_entity_dataset import FolderEntityDataset
from .folder_entity_dataset_data import FolderEntityDatasetData
from .folder_entity_document_index import FolderEntityDocumentIndex
from .folder_entity_document_index_data import FolderEntityDocumentIndexData
from .folder_entity_folder import FolderEntityFolder
from .folder_entity_folder_data import FolderEntityFolderData
from .folder_entity_prompt_sandbox import FolderEntityPromptSandbox
from .folder_entity_prompt_sandbox_data import FolderEntityPromptSandboxData
from .folder_entity_test_suite import FolderEntityTestSuite
from .folder_entity_test_suite_data import FolderEntityTestSuiteData
from .folder_entity_workflow_sandbox import FolderEntityWorkflowSandbox
from .folder_entity_workflow_sandbox_data import FolderEntityWorkflowSandboxData
from .fulfilled_ad_hoc_execute_prompt_event import FulfilledAdHocExecutePromptEvent
from .fulfilled_execute_prompt_event import FulfilledExecutePromptEvent
from .fulfilled_execute_prompt_response import FulfilledExecutePromptResponse
from .fulfilled_execute_workflow_workflow_result_event import FulfilledExecuteWorkflowWorkflowResultEvent
from .fulfilled_prompt_execution_meta import FulfilledPromptExecutionMeta
from .fulfilled_workflow_node_result_event import FulfilledWorkflowNodeResultEvent
from .function_call import FunctionCall
from .function_call_chat_message_content import FunctionCallChatMessageContent
from .function_call_chat_message_content_request import FunctionCallChatMessageContentRequest
from .function_call_chat_message_content_value import FunctionCallChatMessageContentValue
from .function_call_chat_message_content_value_request import FunctionCallChatMessageContentValueRequest
from .function_call_input import FunctionCallInput
from .function_call_prompt_block import FunctionCallPromptBlock
from .function_call_request import FunctionCallRequest
from .function_call_vellum_value import FunctionCallVellumValue
from .function_call_vellum_value_request import FunctionCallVellumValueRequest
from .function_definition import FunctionDefinition
from .generate_options_request import GenerateOptionsRequest
from .generate_request import GenerateRequest
from .generate_response import GenerateResponse
from .generate_result import GenerateResult
from .generate_result_data import GenerateResultData
from .generate_result_error import GenerateResultError
from .generate_stream_response import GenerateStreamResponse
from .generate_stream_result import GenerateStreamResult
from .generate_stream_result_data import GenerateStreamResultData
from .google_vertex_ai_vectorizer_config import GoogleVertexAiVectorizerConfig
from .google_vertex_ai_vectorizer_config_request import GoogleVertexAiVectorizerConfigRequest
from .google_vertex_ai_vectorizer_gemini_embedding_001 import GoogleVertexAiVectorizerGeminiEmbedding001
from .google_vertex_ai_vectorizer_gemini_embedding_001_request import GoogleVertexAiVectorizerGeminiEmbedding001Request
from .google_vertex_ai_vectorizer_text_embedding_004 import GoogleVertexAiVectorizerTextEmbedding004
from .google_vertex_ai_vectorizer_text_embedding_004_request import GoogleVertexAiVectorizerTextEmbedding004Request
from .google_vertex_ai_vectorizer_text_multilingual_embedding_002 import (
    GoogleVertexAiVectorizerTextMultilingualEmbedding002,
)
from .google_vertex_ai_vectorizer_text_multilingual_embedding_002_request import (
    GoogleVertexAiVectorizerTextMultilingualEmbedding002Request,
)
from .hkunlp_instructor_xl_vectorizer import HkunlpInstructorXlVectorizer
from .hkunlp_instructor_xl_vectorizer_request import HkunlpInstructorXlVectorizerRequest
from .image_chat_message_content import ImageChatMessageContent
from .image_chat_message_content_request import ImageChatMessageContentRequest
from .image_prompt_block import ImagePromptBlock
from .image_vellum_value import ImageVellumValue
from .image_vellum_value_request import ImageVellumValueRequest
from .indexing_config_vectorizer import IndexingConfigVectorizer
from .indexing_config_vectorizer_request import IndexingConfigVectorizerRequest
from .indexing_state_enum import IndexingStateEnum
from .initiated_ad_hoc_execute_prompt_event import InitiatedAdHocExecutePromptEvent
from .initiated_execute_prompt_event import InitiatedExecutePromptEvent
from .initiated_prompt_execution_meta import InitiatedPromptExecutionMeta
from .initiated_workflow_node_result_event import InitiatedWorkflowNodeResultEvent
from .instructor_vectorizer_config import InstructorVectorizerConfig
from .instructor_vectorizer_config_request import InstructorVectorizerConfigRequest
from .invoked_port import InvokedPort
from .iteration_state_enum import IterationStateEnum
from .jinja_prompt_block import JinjaPromptBlock
from .json_input import JsonInput
from .json_input_request import JsonInputRequest
from .json_vellum_value import JsonVellumValue
from .json_vellum_value_request import JsonVellumValueRequest
from .logical_operator import LogicalOperator
from .logprobs_enum import LogprobsEnum
from .map_node_result import MapNodeResult
from .map_node_result_data import MapNodeResultData
from .merge_node_result import MergeNodeResult
from .merge_node_result_data import MergeNodeResultData
from .metadata_filter_config_request import MetadataFilterConfigRequest
from .metadata_filter_rule_combinator import MetadataFilterRuleCombinator
from .metadata_filter_rule_request import MetadataFilterRuleRequest
from .metadata_filters_request import MetadataFiltersRequest
from .method_enum import MethodEnum
from .metric_definition_execution import MetricDefinitionExecution
from .metric_definition_history_item import MetricDefinitionHistoryItem
from .metric_definition_input import MetricDefinitionInput
from .metric_node_result import MetricNodeResult
from .ml_model_read import MlModelRead
from .ml_model_usage import MlModelUsage
from .ml_model_usage_wrapper import MlModelUsageWrapper
from .named_scenario_input_chat_history_variable_value_request import NamedScenarioInputChatHistoryVariableValueRequest
from .named_scenario_input_json_variable_value_request import NamedScenarioInputJsonVariableValueRequest
from .named_scenario_input_request import NamedScenarioInputRequest
from .named_scenario_input_string_variable_value_request import NamedScenarioInputStringVariableValueRequest
from .named_test_case_array_variable_value import NamedTestCaseArrayVariableValue
from .named_test_case_array_variable_value_request import NamedTestCaseArrayVariableValueRequest
from .named_test_case_chat_history_variable_value import NamedTestCaseChatHistoryVariableValue
from .named_test_case_chat_history_variable_value_request import NamedTestCaseChatHistoryVariableValueRequest
from .named_test_case_error_variable_value import NamedTestCaseErrorVariableValue
from .named_test_case_error_variable_value_request import NamedTestCaseErrorVariableValueRequest
from .named_test_case_function_call_variable_value import NamedTestCaseFunctionCallVariableValue
from .named_test_case_function_call_variable_value_request import NamedTestCaseFunctionCallVariableValueRequest
from .named_test_case_json_variable_value import NamedTestCaseJsonVariableValue
from .named_test_case_json_variable_value_request import NamedTestCaseJsonVariableValueRequest
from .named_test_case_number_variable_value import NamedTestCaseNumberVariableValue
from .named_test_case_number_variable_value_request import NamedTestCaseNumberVariableValueRequest
from .named_test_case_search_results_variable_value import NamedTestCaseSearchResultsVariableValue
from .named_test_case_search_results_variable_value_request import NamedTestCaseSearchResultsVariableValueRequest
from .named_test_case_string_variable_value import NamedTestCaseStringVariableValue
from .named_test_case_string_variable_value_request import NamedTestCaseStringVariableValueRequest
from .named_test_case_variable_value import NamedTestCaseVariableValue
from .named_test_case_variable_value_request import NamedTestCaseVariableValueRequest
from .new_member_join_behavior_enum import NewMemberJoinBehaviorEnum
from .node_execution_fulfilled_body import NodeExecutionFulfilledBody
from .node_execution_fulfilled_event import NodeExecutionFulfilledEvent
from .node_execution_initiated_body import NodeExecutionInitiatedBody
from .node_execution_initiated_event import NodeExecutionInitiatedEvent
from .node_execution_paused_body import NodeExecutionPausedBody
from .node_execution_paused_event import NodeExecutionPausedEvent
from .node_execution_rejected_body import NodeExecutionRejectedBody
from .node_execution_rejected_event import NodeExecutionRejectedEvent
from .node_execution_resumed_body import NodeExecutionResumedBody
from .node_execution_resumed_event import NodeExecutionResumedEvent
from .node_execution_span import NodeExecutionSpan
from .node_execution_span_attributes import NodeExecutionSpanAttributes
from .node_execution_streaming_body import NodeExecutionStreamingBody
from .node_execution_streaming_event import NodeExecutionStreamingEvent
from .node_input_compiled_array_value import NodeInputCompiledArrayValue
from .node_input_compiled_chat_history_value import NodeInputCompiledChatHistoryValue
from .node_input_compiled_error_value import NodeInputCompiledErrorValue
from .node_input_compiled_function_call_value import NodeInputCompiledFunctionCallValue
from .node_input_compiled_json_value import NodeInputCompiledJsonValue
from .node_input_compiled_number_value import NodeInputCompiledNumberValue
from .node_input_compiled_search_results_value import NodeInputCompiledSearchResultsValue
from .node_input_compiled_secret_value import NodeInputCompiledSecretValue
from .node_input_compiled_string_value import NodeInputCompiledStringValue
from .node_input_variable_compiled_value import NodeInputVariableCompiledValue
from .node_output_compiled_array_value import NodeOutputCompiledArrayValue
from .node_output_compiled_chat_history_value import NodeOutputCompiledChatHistoryValue
from .node_output_compiled_error_value import NodeOutputCompiledErrorValue
from .node_output_compiled_function_call_value import NodeOutputCompiledFunctionCallValue
from .node_output_compiled_json_value import NodeOutputCompiledJsonValue
from .node_output_compiled_number_value import NodeOutputCompiledNumberValue
from .node_output_compiled_search_results_value import NodeOutputCompiledSearchResultsValue
from .node_output_compiled_string_value import NodeOutputCompiledStringValue
from .node_output_compiled_thinking_value import NodeOutputCompiledThinkingValue
from .node_output_compiled_value import NodeOutputCompiledValue
from .node_parent_context import NodeParentContext
from .normalized_log_probs import NormalizedLogProbs
from .normalized_token_log_probs import NormalizedTokenLogProbs
from .number_input import NumberInput
from .number_vellum_value import NumberVellumValue
from .number_vellum_value_request import NumberVellumValueRequest
from .open_ai_vectorizer_config import OpenAiVectorizerConfig
from .open_ai_vectorizer_config_request import OpenAiVectorizerConfigRequest
from .open_ai_vectorizer_text_embedding_3_large import OpenAiVectorizerTextEmbedding3Large
from .open_ai_vectorizer_text_embedding_3_large_request import OpenAiVectorizerTextEmbedding3LargeRequest
from .open_ai_vectorizer_text_embedding_3_small import OpenAiVectorizerTextEmbedding3Small
from .open_ai_vectorizer_text_embedding_3_small_request import OpenAiVectorizerTextEmbedding3SmallRequest
from .open_ai_vectorizer_text_embedding_ada_002 import OpenAiVectorizerTextEmbeddingAda002
from .open_ai_vectorizer_text_embedding_ada_002_request import OpenAiVectorizerTextEmbeddingAda002Request
from .organization_read import OrganizationRead
from .paginated_container_image_read_list import PaginatedContainerImageReadList
from .paginated_deployment_release_tag_read_list import PaginatedDeploymentReleaseTagReadList
from .paginated_document_index_read_list import PaginatedDocumentIndexReadList
from .paginated_folder_entity_list import PaginatedFolderEntityList
from .paginated_slim_deployment_read_list import PaginatedSlimDeploymentReadList
from .paginated_slim_document_list import PaginatedSlimDocumentList
from .paginated_slim_workflow_deployment_list import PaginatedSlimWorkflowDeploymentList
from .paginated_test_suite_run_execution_list import PaginatedTestSuiteRunExecutionList
from .paginated_test_suite_test_case_list import PaginatedTestSuiteTestCaseList
from .paginated_workflow_release_tag_read_list import PaginatedWorkflowReleaseTagReadList
from .paginated_workflow_sandbox_example_list import PaginatedWorkflowSandboxExampleList
from .parent_context import ParentContext
from .pdf_search_result_meta_source import PdfSearchResultMetaSource
from .pdf_search_result_meta_source_request import PdfSearchResultMetaSourceRequest
from .plain_text_prompt_block import PlainTextPromptBlock
from .price import Price
from .processing_failure_reason_enum import ProcessingFailureReasonEnum
from .prompt_block import PromptBlock
from .prompt_block_state import PromptBlockState
from .prompt_deployment_expand_meta_request import PromptDeploymentExpandMetaRequest
from .prompt_deployment_input_request import PromptDeploymentInputRequest
from .prompt_deployment_parent_context import PromptDeploymentParentContext
from .prompt_deployment_release import PromptDeploymentRelease
from .prompt_deployment_release_prompt_deployment import PromptDeploymentReleasePromptDeployment
from .prompt_deployment_release_prompt_version import PromptDeploymentReleasePromptVersion
from .prompt_exec_config import PromptExecConfig
from .prompt_execution_meta import PromptExecutionMeta
from .prompt_node_execution_meta import PromptNodeExecutionMeta
from .prompt_node_result import PromptNodeResult
from .prompt_node_result_data import PromptNodeResultData
from .prompt_output import PromptOutput
from .prompt_parameters import PromptParameters
from .prompt_push_response import PromptPushResponse
from .prompt_request_chat_history_input import PromptRequestChatHistoryInput
from .prompt_request_input import PromptRequestInput
from .prompt_request_json_input import PromptRequestJsonInput
from .prompt_request_string_input import PromptRequestStringInput
from .prompt_settings import PromptSettings
from .prompt_version_build_config_sandbox import PromptVersionBuildConfigSandbox
from .raw_prompt_execution_overrides_request import RawPromptExecutionOverridesRequest
from .reducto_chunker_config import ReductoChunkerConfig
from .reducto_chunker_config_request import ReductoChunkerConfigRequest
from .reducto_chunking import ReductoChunking
from .reducto_chunking_request import ReductoChunkingRequest
from .rejected_ad_hoc_execute_prompt_event import RejectedAdHocExecutePromptEvent
from .rejected_execute_prompt_event import RejectedExecutePromptEvent
from .rejected_execute_prompt_response import RejectedExecutePromptResponse
from .rejected_execute_workflow_workflow_result_event import RejectedExecuteWorkflowWorkflowResultEvent
from .rejected_prompt_execution_meta import RejectedPromptExecutionMeta
from .rejected_workflow_node_result_event import RejectedWorkflowNodeResultEvent
from .release_created_by import ReleaseCreatedBy
from .release_environment import ReleaseEnvironment
from .release_release_tag import ReleaseReleaseTag
from .release_review_reviewer import ReleaseReviewReviewer
from .release_review_state import ReleaseReviewState
from .release_tag_release import ReleaseTagRelease
from .release_tag_source import ReleaseTagSource
from .replace_test_suite_test_case_request import ReplaceTestSuiteTestCaseRequest
from .rich_text_child_block import RichTextChildBlock
from .rich_text_prompt_block import RichTextPromptBlock
from .sandbox_scenario import SandboxScenario
from .scenario_input import ScenarioInput
from .scenario_input_chat_history_variable_value import ScenarioInputChatHistoryVariableValue
from .scenario_input_json_variable_value import ScenarioInputJsonVariableValue
from .scenario_input_string_variable_value import ScenarioInputStringVariableValue
from .search_filters_request import SearchFiltersRequest
from .search_node_result import SearchNodeResult
from .search_node_result_data import SearchNodeResultData
from .search_request_options_request import SearchRequestOptionsRequest
from .search_response import SearchResponse
from .search_result import SearchResult
from .search_result_document import SearchResultDocument
from .search_result_document_request import SearchResultDocumentRequest
from .search_result_merging_request import SearchResultMergingRequest
from .search_result_meta import SearchResultMeta
from .search_result_meta_request import SearchResultMetaRequest
from .search_result_request import SearchResultRequest
from .search_results_input import SearchResultsInput
from .search_results_vellum_value import SearchResultsVellumValue
from .search_results_vellum_value_request import SearchResultsVellumValueRequest
from .search_weights_request import SearchWeightsRequest
from .secret_type_enum import SecretTypeEnum
from .sentence_chunker_config import SentenceChunkerConfig
from .sentence_chunker_config_request import SentenceChunkerConfigRequest
from .sentence_chunking import SentenceChunking
from .sentence_chunking_request import SentenceChunkingRequest
from .slim_deployment_read import SlimDeploymentRead
from .slim_document import SlimDocument
from .slim_document_document_to_document_index import SlimDocumentDocumentToDocumentIndex
from .slim_release_review import SlimReleaseReview
from .slim_workflow_deployment import SlimWorkflowDeployment
from .slim_workflow_execution_read import SlimWorkflowExecutionRead
from .span_link import SpanLink
from .span_link_type_enum import SpanLinkTypeEnum
from .streaming_ad_hoc_execute_prompt_event import StreamingAdHocExecutePromptEvent
from .streaming_execute_prompt_event import StreamingExecutePromptEvent
from .streaming_prompt_execution_meta import StreamingPromptExecutionMeta
from .streaming_workflow_node_result_event import StreamingWorkflowNodeResultEvent
from .string_chat_message_content import StringChatMessageContent
from .string_chat_message_content_request import StringChatMessageContentRequest
from .string_input import StringInput
from .string_input_request import StringInputRequest
from .string_vellum_value import StringVellumValue
from .string_vellum_value_request import StringVellumValueRequest
from .submit_completion_actual_request import SubmitCompletionActualRequest
from .submit_workflow_execution_actual_request import SubmitWorkflowExecutionActualRequest
from .subworkflow_node_result import SubworkflowNodeResult
from .subworkflow_node_result_data import SubworkflowNodeResultData
from .templating_node_array_result import TemplatingNodeArrayResult
from .templating_node_chat_history_result import TemplatingNodeChatHistoryResult
from .templating_node_error_result import TemplatingNodeErrorResult
from .templating_node_function_call_result import TemplatingNodeFunctionCallResult
from .templating_node_json_result import TemplatingNodeJsonResult
from .templating_node_number_result import TemplatingNodeNumberResult
from .templating_node_result import TemplatingNodeResult
from .templating_node_result_data import TemplatingNodeResultData
from .templating_node_result_output import TemplatingNodeResultOutput
from .templating_node_search_results_result import TemplatingNodeSearchResultsResult
from .templating_node_string_result import TemplatingNodeStringResult
from .terminal_node_array_result import TerminalNodeArrayResult
from .terminal_node_chat_history_result import TerminalNodeChatHistoryResult
from .terminal_node_error_result import TerminalNodeErrorResult
from .terminal_node_function_call_result import TerminalNodeFunctionCallResult
from .terminal_node_json_result import TerminalNodeJsonResult
from .terminal_node_number_result import TerminalNodeNumberResult
from .terminal_node_result import TerminalNodeResult
from .terminal_node_result_data import TerminalNodeResultData
from .terminal_node_result_output import TerminalNodeResultOutput
from .terminal_node_search_results_result import TerminalNodeSearchResultsResult
from .terminal_node_string_result import TerminalNodeStringResult
from .test_case_array_variable_value import TestCaseArrayVariableValue
from .test_case_chat_history_variable_value import TestCaseChatHistoryVariableValue
from .test_case_error_variable_value import TestCaseErrorVariableValue
from .test_case_function_call_variable_value import TestCaseFunctionCallVariableValue
from .test_case_json_variable_value import TestCaseJsonVariableValue
from .test_case_number_variable_value import TestCaseNumberVariableValue
from .test_case_search_results_variable_value import TestCaseSearchResultsVariableValue
from .test_case_string_variable_value import TestCaseStringVariableValue
from .test_case_variable_value import TestCaseVariableValue
from .test_suite_run_deployment_release_tag_exec_config import TestSuiteRunDeploymentReleaseTagExecConfig
from .test_suite_run_deployment_release_tag_exec_config_data import TestSuiteRunDeploymentReleaseTagExecConfigData
from .test_suite_run_deployment_release_tag_exec_config_data_request import (
    TestSuiteRunDeploymentReleaseTagExecConfigDataRequest,
)
from .test_suite_run_deployment_release_tag_exec_config_request import TestSuiteRunDeploymentReleaseTagExecConfigRequest
from .test_suite_run_exec_config import TestSuiteRunExecConfig
from .test_suite_run_exec_config_request import TestSuiteRunExecConfigRequest
from .test_suite_run_execution import TestSuiteRunExecution
from .test_suite_run_execution_array_output import TestSuiteRunExecutionArrayOutput
from .test_suite_run_execution_chat_history_output import TestSuiteRunExecutionChatHistoryOutput
from .test_suite_run_execution_error_output import TestSuiteRunExecutionErrorOutput
from .test_suite_run_execution_function_call_output import TestSuiteRunExecutionFunctionCallOutput
from .test_suite_run_execution_json_output import TestSuiteRunExecutionJsonOutput
from .test_suite_run_execution_metric_definition import TestSuiteRunExecutionMetricDefinition
from .test_suite_run_execution_metric_result import TestSuiteRunExecutionMetricResult
from .test_suite_run_execution_number_output import TestSuiteRunExecutionNumberOutput
from .test_suite_run_execution_output import TestSuiteRunExecutionOutput
from .test_suite_run_execution_search_results_output import TestSuiteRunExecutionSearchResultsOutput
from .test_suite_run_execution_string_output import TestSuiteRunExecutionStringOutput
from .test_suite_run_external_exec_config import TestSuiteRunExternalExecConfig
from .test_suite_run_external_exec_config_data import TestSuiteRunExternalExecConfigData
from .test_suite_run_external_exec_config_data_request import TestSuiteRunExternalExecConfigDataRequest
from .test_suite_run_external_exec_config_request import TestSuiteRunExternalExecConfigRequest
from .test_suite_run_metric_array_output import TestSuiteRunMetricArrayOutput
from .test_suite_run_metric_error_output import TestSuiteRunMetricErrorOutput
from .test_suite_run_metric_json_output import TestSuiteRunMetricJsonOutput
from .test_suite_run_metric_number_output import TestSuiteRunMetricNumberOutput
from .test_suite_run_metric_output import TestSuiteRunMetricOutput
from .test_suite_run_metric_string_output import TestSuiteRunMetricStringOutput
from .test_suite_run_progress import TestSuiteRunProgress
from .test_suite_run_prompt_sandbox_exec_config_data_request import TestSuiteRunPromptSandboxExecConfigDataRequest
from .test_suite_run_prompt_sandbox_exec_config_request import TestSuiteRunPromptSandboxExecConfigRequest
from .test_suite_run_prompt_sandbox_history_item_exec_config import TestSuiteRunPromptSandboxHistoryItemExecConfig
from .test_suite_run_prompt_sandbox_history_item_exec_config_data import (
    TestSuiteRunPromptSandboxHistoryItemExecConfigData,
)
from .test_suite_run_prompt_sandbox_history_item_exec_config_data_request import (
    TestSuiteRunPromptSandboxHistoryItemExecConfigDataRequest,
)
from .test_suite_run_prompt_sandbox_history_item_exec_config_request import (
    TestSuiteRunPromptSandboxHistoryItemExecConfigRequest,
)
from .test_suite_run_read import TestSuiteRunRead
from .test_suite_run_state import TestSuiteRunState
from .test_suite_run_test_suite import TestSuiteRunTestSuite
from .test_suite_run_workflow_release_tag_exec_config import TestSuiteRunWorkflowReleaseTagExecConfig
from .test_suite_run_workflow_release_tag_exec_config_data import TestSuiteRunWorkflowReleaseTagExecConfigData
from .test_suite_run_workflow_release_tag_exec_config_data_request import (
    TestSuiteRunWorkflowReleaseTagExecConfigDataRequest,
)
from .test_suite_run_workflow_release_tag_exec_config_request import TestSuiteRunWorkflowReleaseTagExecConfigRequest
from .test_suite_run_workflow_sandbox_exec_config_data_request import TestSuiteRunWorkflowSandboxExecConfigDataRequest
from .test_suite_run_workflow_sandbox_exec_config_request import TestSuiteRunWorkflowSandboxExecConfigRequest
from .test_suite_run_workflow_sandbox_history_item_exec_config import TestSuiteRunWorkflowSandboxHistoryItemExecConfig
from .test_suite_run_workflow_sandbox_history_item_exec_config_data import (
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigData,
)
from .test_suite_run_workflow_sandbox_history_item_exec_config_data_request import (
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigDataRequest,
)
from .test_suite_run_workflow_sandbox_history_item_exec_config_request import (
    TestSuiteRunWorkflowSandboxHistoryItemExecConfigRequest,
)
from .test_suite_test_case import TestSuiteTestCase
from .test_suite_test_case_bulk_operation_request import TestSuiteTestCaseBulkOperationRequest
from .test_suite_test_case_bulk_result import TestSuiteTestCaseBulkResult
from .test_suite_test_case_create_bulk_operation_request import TestSuiteTestCaseCreateBulkOperationRequest
from .test_suite_test_case_created_bulk_result import TestSuiteTestCaseCreatedBulkResult
from .test_suite_test_case_created_bulk_result_data import TestSuiteTestCaseCreatedBulkResultData
from .test_suite_test_case_delete_bulk_operation_data_request import TestSuiteTestCaseDeleteBulkOperationDataRequest
from .test_suite_test_case_delete_bulk_operation_request import TestSuiteTestCaseDeleteBulkOperationRequest
from .test_suite_test_case_deleted_bulk_result import TestSuiteTestCaseDeletedBulkResult
from .test_suite_test_case_deleted_bulk_result_data import TestSuiteTestCaseDeletedBulkResultData
from .test_suite_test_case_rejected_bulk_result import TestSuiteTestCaseRejectedBulkResult
from .test_suite_test_case_replace_bulk_operation_request import TestSuiteTestCaseReplaceBulkOperationRequest
from .test_suite_test_case_replaced_bulk_result import TestSuiteTestCaseReplacedBulkResult
from .test_suite_test_case_replaced_bulk_result_data import TestSuiteTestCaseReplacedBulkResultData
from .test_suite_test_case_upsert_bulk_operation_request import TestSuiteTestCaseUpsertBulkOperationRequest
from .thinking_vellum_value import ThinkingVellumValue
from .thinking_vellum_value_request import ThinkingVellumValueRequest
from .token_overlapping_window_chunker_config import TokenOverlappingWindowChunkerConfig
from .token_overlapping_window_chunker_config_request import TokenOverlappingWindowChunkerConfigRequest
from .token_overlapping_window_chunking import TokenOverlappingWindowChunking
from .token_overlapping_window_chunking_request import TokenOverlappingWindowChunkingRequest
from .unit_enum import UnitEnum
from .upload_document_response import UploadDocumentResponse
from .upsert_test_suite_test_case_request import UpsertTestSuiteTestCaseRequest
from .variable_prompt_block import VariablePromptBlock
from .vellum_audio import VellumAudio
from .vellum_audio_request import VellumAudioRequest
from .vellum_code_resource_definition import VellumCodeResourceDefinition
from .vellum_document import VellumDocument
from .vellum_document_request import VellumDocumentRequest
from .vellum_error import VellumError
from .vellum_error_code_enum import VellumErrorCodeEnum
from .vellum_error_request import VellumErrorRequest
from .vellum_image import VellumImage
from .vellum_image_request import VellumImageRequest
from .vellum_node_execution_event import VellumNodeExecutionEvent
from .vellum_sdk_error import VellumSdkError
from .vellum_sdk_error_code_enum import VellumSdkErrorCodeEnum
from .vellum_secret import VellumSecret
from .vellum_span import VellumSpan
from .vellum_value import VellumValue
from .vellum_value_logical_condition_group_request import VellumValueLogicalConditionGroupRequest
from .vellum_value_logical_condition_request import VellumValueLogicalConditionRequest
from .vellum_value_logical_expression_request import VellumValueLogicalExpressionRequest
from .vellum_value_request import VellumValueRequest
from .vellum_variable import VellumVariable
from .vellum_variable_extensions import VellumVariableExtensions
from .vellum_variable_type import VellumVariableType
from .vellum_video import VellumVideo
from .vellum_video_request import VellumVideoRequest
from .vellum_workflow_execution_event import VellumWorkflowExecutionEvent
from .video_chat_message_content import VideoChatMessageContent
from .video_chat_message_content_request import VideoChatMessageContentRequest
from .video_prompt_block import VideoPromptBlock
from .video_vellum_value import VideoVellumValue
from .video_vellum_value_request import VideoVellumValueRequest
from .workflow_deployment_event_executions_response import WorkflowDeploymentEventExecutionsResponse
from .workflow_deployment_history_item import WorkflowDeploymentHistoryItem
from .workflow_deployment_parent_context import WorkflowDeploymentParentContext
from .workflow_deployment_read import WorkflowDeploymentRead
from .workflow_deployment_release import WorkflowDeploymentRelease
from .workflow_deployment_release_workflow_deployment import WorkflowDeploymentReleaseWorkflowDeployment
from .workflow_deployment_release_workflow_version import WorkflowDeploymentReleaseWorkflowVersion
from .workflow_error import WorkflowError
from .workflow_event import WorkflowEvent
from .workflow_event_error import WorkflowEventError
from .workflow_event_execution_read import WorkflowEventExecutionRead
from .workflow_execution_actual import WorkflowExecutionActual
from .workflow_execution_actual_chat_history_request import WorkflowExecutionActualChatHistoryRequest
from .workflow_execution_actual_json_request import WorkflowExecutionActualJsonRequest
from .workflow_execution_actual_string_request import WorkflowExecutionActualStringRequest
from .workflow_execution_detail import WorkflowExecutionDetail
from .workflow_execution_event_error_code import WorkflowExecutionEventErrorCode
from .workflow_execution_event_type import WorkflowExecutionEventType
from .workflow_execution_fulfilled_body import WorkflowExecutionFulfilledBody
from .workflow_execution_fulfilled_event import WorkflowExecutionFulfilledEvent
from .workflow_execution_initiated_body import WorkflowExecutionInitiatedBody
from .workflow_execution_initiated_event import WorkflowExecutionInitiatedEvent
from .workflow_execution_node_result_event import WorkflowExecutionNodeResultEvent
from .workflow_execution_paused_body import WorkflowExecutionPausedBody
from .workflow_execution_paused_event import WorkflowExecutionPausedEvent
from .workflow_execution_rejected_body import WorkflowExecutionRejectedBody
from .workflow_execution_rejected_event import WorkflowExecutionRejectedEvent
from .workflow_execution_resumed_body import WorkflowExecutionResumedBody
from .workflow_execution_resumed_event import WorkflowExecutionResumedEvent
from .workflow_execution_snapshotted_body import WorkflowExecutionSnapshottedBody
from .workflow_execution_snapshotted_event import WorkflowExecutionSnapshottedEvent
from .workflow_execution_span import WorkflowExecutionSpan
from .workflow_execution_span_attributes import WorkflowExecutionSpanAttributes
from .workflow_execution_streaming_body import WorkflowExecutionStreamingBody
from .workflow_execution_streaming_event import WorkflowExecutionStreamingEvent
from .workflow_execution_usage_calculation_error import WorkflowExecutionUsageCalculationError
from .workflow_execution_usage_calculation_error_code_enum import WorkflowExecutionUsageCalculationErrorCodeEnum
from .workflow_execution_usage_calculation_fulfilled_body import WorkflowExecutionUsageCalculationFulfilledBody
from .workflow_execution_usage_result import WorkflowExecutionUsageResult
from .workflow_execution_view_online_eval_metric_result import WorkflowExecutionViewOnlineEvalMetricResult
from .workflow_execution_workflow_result_event import WorkflowExecutionWorkflowResultEvent
from .workflow_expand_meta_request import WorkflowExpandMetaRequest
from .workflow_initialization_error import WorkflowInitializationError
from .workflow_node_result_data import WorkflowNodeResultData
from .workflow_node_result_event import WorkflowNodeResultEvent
from .workflow_node_result_event_state import WorkflowNodeResultEventState
from .workflow_output import WorkflowOutput
from .workflow_output_array import WorkflowOutputArray
from .workflow_output_chat_history import WorkflowOutputChatHistory
from .workflow_output_error import WorkflowOutputError
from .workflow_output_function_call import WorkflowOutputFunctionCall
from .workflow_output_image import WorkflowOutputImage
from .workflow_output_json import WorkflowOutputJson
from .workflow_output_number import WorkflowOutputNumber
from .workflow_output_search_results import WorkflowOutputSearchResults
from .workflow_output_string import WorkflowOutputString
from .workflow_parent_context import WorkflowParentContext
from .workflow_push_deployment_config_request import WorkflowPushDeploymentConfigRequest
from .workflow_push_exec_config import WorkflowPushExecConfig
from .workflow_push_response import WorkflowPushResponse
from .workflow_release_tag_read import WorkflowReleaseTagRead
from .workflow_release_tag_workflow_deployment_history_item import WorkflowReleaseTagWorkflowDeploymentHistoryItem
from .workflow_request_chat_history_input_request import WorkflowRequestChatHistoryInputRequest
from .workflow_request_input_request import WorkflowRequestInputRequest
from .workflow_request_json_input_request import WorkflowRequestJsonInputRequest
from .workflow_request_number_input_request import WorkflowRequestNumberInputRequest
from .workflow_request_string_input_request import WorkflowRequestStringInputRequest
from .workflow_result_event import WorkflowResultEvent
from .workflow_result_event_output_data import WorkflowResultEventOutputData
from .workflow_result_event_output_data_array import WorkflowResultEventOutputDataArray
from .workflow_result_event_output_data_chat_history import WorkflowResultEventOutputDataChatHistory
from .workflow_result_event_output_data_error import WorkflowResultEventOutputDataError
from .workflow_result_event_output_data_function_call import WorkflowResultEventOutputDataFunctionCall
from .workflow_result_event_output_data_json import WorkflowResultEventOutputDataJson
from .workflow_result_event_output_data_number import WorkflowResultEventOutputDataNumber
from .workflow_result_event_output_data_search_results import WorkflowResultEventOutputDataSearchResults
from .workflow_result_event_output_data_string import WorkflowResultEventOutputDataString
from .workflow_sandbox_example import WorkflowSandboxExample
from .workflow_sandbox_parent_context import WorkflowSandboxParentContext
from .workflow_stream_event import WorkflowStreamEvent
from .workspace_read import WorkspaceRead
from .workspace_secret_read import WorkspaceSecretRead

__all__ = [
    "AdHocExecutePromptEvent",
    "AdHocExpandMeta",
    "AdHocFulfilledPromptExecutionMeta",
    "AdHocInitiatedPromptExecutionMeta",
    "AdHocRejectedPromptExecutionMeta",
    "AdHocStreamingPromptExecutionMeta",
    "AddOpenaiApiKeyEnum",
    "ApiNodeResult",
    "ApiNodeResultData",
    "ApiRequestParentContext",
    "ApiVersionEnum",
    "ArrayChatMessageContent",
    "ArrayChatMessageContentItem",
    "ArrayChatMessageContentItemRequest",
    "ArrayChatMessageContentRequest",
    "ArrayInput",
    "ArrayVellumValue",
    "ArrayVellumValueRequest",
    "AudioChatMessageContent",
    "AudioChatMessageContentRequest",
    "AudioPromptBlock",
    "AudioVellumValue",
    "AudioVellumValueRequest",
    "BaseOutput",
    "BasicVectorizerIntfloatMultilingualE5Large",
    "BasicVectorizerIntfloatMultilingualE5LargeRequest",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseCosV1Request",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1",
    "BasicVectorizerSentenceTransformersMultiQaMpnetBaseDotV1Request",
    "BuildStatusEnum",
    "ChatHistoryInput",
    "ChatHistoryInputRequest",
    "ChatHistoryVellumValue",
    "ChatHistoryVellumValueRequest",
    "ChatMessage",
    "ChatMessageContent",
    "ChatMessageContentRequest",
    "ChatMessagePromptBlock",
    "ChatMessageRequest",
    "ChatMessageRole",
    "CodeExecutionNodeArrayResult",
    "CodeExecutionNodeChatHistoryResult",
    "CodeExecutionNodeErrorResult",
    "CodeExecutionNodeFunctionCallResult",
    "CodeExecutionNodeJsonResult",
    "CodeExecutionNodeNumberResult",
    "CodeExecutionNodeResult",
    "CodeExecutionNodeResultData",
    "CodeExecutionNodeResultOutput",
    "CodeExecutionNodeSearchResultsResult",
    "CodeExecutionNodeStringResult",
    "CodeExecutionPackage",
    "CodeExecutionRuntime",
    "CodeExecutorInput",
    "CodeExecutorResponse",
    "CodeExecutorSecretInput",
    "CodeResourceDefinition",
    "CompilePromptDeploymentExpandMetaRequest",
    "CompilePromptMeta",
    "ComponentsSchemasPdfSearchResultMetaSource",
    "ComponentsSchemasPdfSearchResultMetaSourceRequest",
    "ComponentsSchemasPromptVersionBuildConfigSandbox",
    "ConditionCombinator",
    "ConditionalNodeResult",
    "ConditionalNodeResultData",
    "ContainerImageBuildConfig",
    "ContainerImageContainerImageTag",
    "ContainerImageRead",
    "CreateTestSuiteTestCaseRequest",
    "DeploymentHistoryItem",
    "DeploymentProviderPayloadResponse",
    "DeploymentProviderPayloadResponsePayload",
    "DeploymentRead",
    "DeploymentReleaseTagDeploymentHistoryItem",
    "DeploymentReleaseTagRead",
    "DockerServiceToken",
    "DocumentChatMessageContent",
    "DocumentChatMessageContentRequest",
    "DocumentDocumentToDocumentIndex",
    "DocumentIndexChunking",
    "DocumentIndexChunkingRequest",
    "DocumentIndexIndexingConfig",
    "DocumentIndexIndexingConfigRequest",
    "DocumentIndexRead",
    "DocumentProcessingState",
    "DocumentPromptBlock",
    "DocumentRead",
    "DocumentStatus",
    "DocumentVellumValue",
    "DocumentVellumValueRequest",
    "EnrichedNormalizedCompletion",
    "EntityStatus",
    "EntityVisibility",
    "EnvironmentEnum",
    "EphemeralPromptCacheConfig",
    "EphemeralPromptCacheConfigTypeEnum",
    "ErrorDetailResponse",
    "ErrorInput",
    "ErrorVellumValue",
    "ErrorVellumValueRequest",
    "EventCreateResponse",
    "ExecuteApiRequestBearerToken",
    "ExecuteApiRequestBody",
    "ExecuteApiRequestHeadersValue",
    "ExecuteApiResponse",
    "ExecuteApiResponseJson",
    "ExecutePromptEvent",
    "ExecutePromptResponse",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowWorkflowResultEvent",
    "ExecutionArrayVellumValue",
    "ExecutionChatHistoryVellumValue",
    "ExecutionErrorVellumValue",
    "ExecutionFunctionCallVellumValue",
    "ExecutionJsonVellumValue",
    "ExecutionNumberVellumValue",
    "ExecutionSearchResultsVellumValue",
    "ExecutionStringVellumValue",
    "ExecutionThinkingVellumValue",
    "ExecutionVellumValue",
    "ExternalInputDescriptor",
    "ExternalParentContext",
    "ExternalTestCaseExecution",
    "ExternalTestCaseExecutionRequest",
    "FastEmbedVectorizerBaaiBgeSmallEnV15",
    "FastEmbedVectorizerBaaiBgeSmallEnV15Request",
    "FinishReasonEnum",
    "FolderEntity",
    "FolderEntityDataset",
    "FolderEntityDatasetData",
    "FolderEntityDocumentIndex",
    "FolderEntityDocumentIndexData",
    "FolderEntityFolder",
    "FolderEntityFolderData",
    "FolderEntityPromptSandbox",
    "FolderEntityPromptSandboxData",
    "FolderEntityTestSuite",
    "FolderEntityTestSuiteData",
    "FolderEntityWorkflowSandbox",
    "FolderEntityWorkflowSandboxData",
    "FulfilledAdHocExecutePromptEvent",
    "FulfilledExecutePromptEvent",
    "FulfilledExecutePromptResponse",
    "FulfilledExecuteWorkflowWorkflowResultEvent",
    "FulfilledPromptExecutionMeta",
    "FulfilledWorkflowNodeResultEvent",
    "FunctionCall",
    "FunctionCallChatMessageContent",
    "FunctionCallChatMessageContentRequest",
    "FunctionCallChatMessageContentValue",
    "FunctionCallChatMessageContentValueRequest",
    "FunctionCallInput",
    "FunctionCallPromptBlock",
    "FunctionCallRequest",
    "FunctionCallVellumValue",
    "FunctionCallVellumValueRequest",
    "FunctionDefinition",
    "GenerateOptionsRequest",
    "GenerateRequest",
    "GenerateResponse",
    "GenerateResult",
    "GenerateResultData",
    "GenerateResultError",
    "GenerateStreamResponse",
    "GenerateStreamResult",
    "GenerateStreamResultData",
    "GoogleVertexAiVectorizerConfig",
    "GoogleVertexAiVectorizerConfigRequest",
    "GoogleVertexAiVectorizerGeminiEmbedding001",
    "GoogleVertexAiVectorizerGeminiEmbedding001Request",
    "GoogleVertexAiVectorizerTextEmbedding004",
    "GoogleVertexAiVectorizerTextEmbedding004Request",
    "GoogleVertexAiVectorizerTextMultilingualEmbedding002",
    "GoogleVertexAiVectorizerTextMultilingualEmbedding002Request",
    "HkunlpInstructorXlVectorizer",
    "HkunlpInstructorXlVectorizerRequest",
    "ImageChatMessageContent",
    "ImageChatMessageContentRequest",
    "ImagePromptBlock",
    "ImageVellumValue",
    "ImageVellumValueRequest",
    "IndexingConfigVectorizer",
    "IndexingConfigVectorizerRequest",
    "IndexingStateEnum",
    "InitiatedAdHocExecutePromptEvent",
    "InitiatedExecutePromptEvent",
    "InitiatedPromptExecutionMeta",
    "InitiatedWorkflowNodeResultEvent",
    "InstructorVectorizerConfig",
    "InstructorVectorizerConfigRequest",
    "InvokedPort",
    "IterationStateEnum",
    "JinjaPromptBlock",
    "JsonInput",
    "JsonInputRequest",
    "JsonVellumValue",
    "JsonVellumValueRequest",
    "LogicalOperator",
    "LogprobsEnum",
    "MapNodeResult",
    "MapNodeResultData",
    "MergeNodeResult",
    "MergeNodeResultData",
    "MetadataFilterConfigRequest",
    "MetadataFilterRuleCombinator",
    "MetadataFilterRuleRequest",
    "MetadataFiltersRequest",
    "MethodEnum",
    "MetricDefinitionExecution",
    "MetricDefinitionHistoryItem",
    "MetricDefinitionInput",
    "MetricNodeResult",
    "MlModelRead",
    "MlModelUsage",
    "MlModelUsageWrapper",
    "NamedScenarioInputChatHistoryVariableValueRequest",
    "NamedScenarioInputJsonVariableValueRequest",
    "NamedScenarioInputRequest",
    "NamedScenarioInputStringVariableValueRequest",
    "NamedTestCaseArrayVariableValue",
    "NamedTestCaseArrayVariableValueRequest",
    "NamedTestCaseChatHistoryVariableValue",
    "NamedTestCaseChatHistoryVariableValueRequest",
    "NamedTestCaseErrorVariableValue",
    "NamedTestCaseErrorVariableValueRequest",
    "NamedTestCaseFunctionCallVariableValue",
    "NamedTestCaseFunctionCallVariableValueRequest",
    "NamedTestCaseJsonVariableValue",
    "NamedTestCaseJsonVariableValueRequest",
    "NamedTestCaseNumberVariableValue",
    "NamedTestCaseNumberVariableValueRequest",
    "NamedTestCaseSearchResultsVariableValue",
    "NamedTestCaseSearchResultsVariableValueRequest",
    "NamedTestCaseStringVariableValue",
    "NamedTestCaseStringVariableValueRequest",
    "NamedTestCaseVariableValue",
    "NamedTestCaseVariableValueRequest",
    "NewMemberJoinBehaviorEnum",
    "NodeExecutionFulfilledBody",
    "NodeExecutionFulfilledEvent",
    "NodeExecutionInitiatedBody",
    "NodeExecutionInitiatedEvent",
    "NodeExecutionPausedBody",
    "NodeExecutionPausedEvent",
    "NodeExecutionRejectedBody",
    "NodeExecutionRejectedEvent",
    "NodeExecutionResumedBody",
    "NodeExecutionResumedEvent",
    "NodeExecutionSpan",
    "NodeExecutionSpanAttributes",
    "NodeExecutionStreamingBody",
    "NodeExecutionStreamingEvent",
    "NodeInputCompiledArrayValue",
    "NodeInputCompiledChatHistoryValue",
    "NodeInputCompiledErrorValue",
    "NodeInputCompiledFunctionCallValue",
    "NodeInputCompiledJsonValue",
    "NodeInputCompiledNumberValue",
    "NodeInputCompiledSearchResultsValue",
    "NodeInputCompiledSecretValue",
    "NodeInputCompiledStringValue",
    "NodeInputVariableCompiledValue",
    "NodeOutputCompiledArrayValue",
    "NodeOutputCompiledChatHistoryValue",
    "NodeOutputCompiledErrorValue",
    "NodeOutputCompiledFunctionCallValue",
    "NodeOutputCompiledJsonValue",
    "NodeOutputCompiledNumberValue",
    "NodeOutputCompiledSearchResultsValue",
    "NodeOutputCompiledStringValue",
    "NodeOutputCompiledThinkingValue",
    "NodeOutputCompiledValue",
    "NodeParentContext",
    "NormalizedLogProbs",
    "NormalizedTokenLogProbs",
    "NumberInput",
    "NumberVellumValue",
    "NumberVellumValueRequest",
    "OpenAiVectorizerConfig",
    "OpenAiVectorizerConfigRequest",
    "OpenAiVectorizerTextEmbedding3Large",
    "OpenAiVectorizerTextEmbedding3LargeRequest",
    "OpenAiVectorizerTextEmbedding3Small",
    "OpenAiVectorizerTextEmbedding3SmallRequest",
    "OpenAiVectorizerTextEmbeddingAda002",
    "OpenAiVectorizerTextEmbeddingAda002Request",
    "OrganizationRead",
    "PaginatedContainerImageReadList",
    "PaginatedDeploymentReleaseTagReadList",
    "PaginatedDocumentIndexReadList",
    "PaginatedFolderEntityList",
    "PaginatedSlimDeploymentReadList",
    "PaginatedSlimDocumentList",
    "PaginatedSlimWorkflowDeploymentList",
    "PaginatedTestSuiteRunExecutionList",
    "PaginatedTestSuiteTestCaseList",
    "PaginatedWorkflowReleaseTagReadList",
    "PaginatedWorkflowSandboxExampleList",
    "ParentContext",
    "PdfSearchResultMetaSource",
    "PdfSearchResultMetaSourceRequest",
    "PlainTextPromptBlock",
    "Price",
    "ProcessingFailureReasonEnum",
    "PromptBlock",
    "PromptBlockState",
    "PromptDeploymentExpandMetaRequest",
    "PromptDeploymentInputRequest",
    "PromptDeploymentParentContext",
    "PromptDeploymentRelease",
    "PromptDeploymentReleasePromptDeployment",
    "PromptDeploymentReleasePromptVersion",
    "PromptExecConfig",
    "PromptExecutionMeta",
    "PromptNodeExecutionMeta",
    "PromptNodeResult",
    "PromptNodeResultData",
    "PromptOutput",
    "PromptParameters",
    "PromptPushResponse",
    "PromptRequestChatHistoryInput",
    "PromptRequestInput",
    "PromptRequestJsonInput",
    "PromptRequestStringInput",
    "PromptSettings",
    "PromptVersionBuildConfigSandbox",
    "RawPromptExecutionOverridesRequest",
    "ReductoChunkerConfig",
    "ReductoChunkerConfigRequest",
    "ReductoChunking",
    "ReductoChunkingRequest",
    "RejectedAdHocExecutePromptEvent",
    "RejectedExecutePromptEvent",
    "RejectedExecutePromptResponse",
    "RejectedExecuteWorkflowWorkflowResultEvent",
    "RejectedPromptExecutionMeta",
    "RejectedWorkflowNodeResultEvent",
    "ReleaseCreatedBy",
    "ReleaseEnvironment",
    "ReleaseReleaseTag",
    "ReleaseReviewReviewer",
    "ReleaseReviewState",
    "ReleaseTagRelease",
    "ReleaseTagSource",
    "ReplaceTestSuiteTestCaseRequest",
    "RichTextChildBlock",
    "RichTextPromptBlock",
    "SandboxScenario",
    "ScenarioInput",
    "ScenarioInputChatHistoryVariableValue",
    "ScenarioInputJsonVariableValue",
    "ScenarioInputStringVariableValue",
    "SearchFiltersRequest",
    "SearchNodeResult",
    "SearchNodeResultData",
    "SearchRequestOptionsRequest",
    "SearchResponse",
    "SearchResult",
    "SearchResultDocument",
    "SearchResultDocumentRequest",
    "SearchResultMergingRequest",
    "SearchResultMeta",
    "SearchResultMetaRequest",
    "SearchResultRequest",
    "SearchResultsInput",
    "SearchResultsVellumValue",
    "SearchResultsVellumValueRequest",
    "SearchWeightsRequest",
    "SecretTypeEnum",
    "SentenceChunkerConfig",
    "SentenceChunkerConfigRequest",
    "SentenceChunking",
    "SentenceChunkingRequest",
    "SlimDeploymentRead",
    "SlimDocument",
    "SlimDocumentDocumentToDocumentIndex",
    "SlimReleaseReview",
    "SlimWorkflowDeployment",
    "SlimWorkflowExecutionRead",
    "SpanLink",
    "SpanLinkTypeEnum",
    "StreamingAdHocExecutePromptEvent",
    "StreamingExecutePromptEvent",
    "StreamingPromptExecutionMeta",
    "StreamingWorkflowNodeResultEvent",
    "StringChatMessageContent",
    "StringChatMessageContentRequest",
    "StringInput",
    "StringInputRequest",
    "StringVellumValue",
    "StringVellumValueRequest",
    "SubmitCompletionActualRequest",
    "SubmitWorkflowExecutionActualRequest",
    "SubworkflowNodeResult",
    "SubworkflowNodeResultData",
    "TemplatingNodeArrayResult",
    "TemplatingNodeChatHistoryResult",
    "TemplatingNodeErrorResult",
    "TemplatingNodeFunctionCallResult",
    "TemplatingNodeJsonResult",
    "TemplatingNodeNumberResult",
    "TemplatingNodeResult",
    "TemplatingNodeResultData",
    "TemplatingNodeResultOutput",
    "TemplatingNodeSearchResultsResult",
    "TemplatingNodeStringResult",
    "TerminalNodeArrayResult",
    "TerminalNodeChatHistoryResult",
    "TerminalNodeErrorResult",
    "TerminalNodeFunctionCallResult",
    "TerminalNodeJsonResult",
    "TerminalNodeNumberResult",
    "TerminalNodeResult",
    "TerminalNodeResultData",
    "TerminalNodeResultOutput",
    "TerminalNodeSearchResultsResult",
    "TerminalNodeStringResult",
    "TestCaseArrayVariableValue",
    "TestCaseChatHistoryVariableValue",
    "TestCaseErrorVariableValue",
    "TestCaseFunctionCallVariableValue",
    "TestCaseJsonVariableValue",
    "TestCaseNumberVariableValue",
    "TestCaseSearchResultsVariableValue",
    "TestCaseStringVariableValue",
    "TestCaseVariableValue",
    "TestSuiteRunDeploymentReleaseTagExecConfig",
    "TestSuiteRunDeploymentReleaseTagExecConfigData",
    "TestSuiteRunDeploymentReleaseTagExecConfigDataRequest",
    "TestSuiteRunDeploymentReleaseTagExecConfigRequest",
    "TestSuiteRunExecConfig",
    "TestSuiteRunExecConfigRequest",
    "TestSuiteRunExecution",
    "TestSuiteRunExecutionArrayOutput",
    "TestSuiteRunExecutionChatHistoryOutput",
    "TestSuiteRunExecutionErrorOutput",
    "TestSuiteRunExecutionFunctionCallOutput",
    "TestSuiteRunExecutionJsonOutput",
    "TestSuiteRunExecutionMetricDefinition",
    "TestSuiteRunExecutionMetricResult",
    "TestSuiteRunExecutionNumberOutput",
    "TestSuiteRunExecutionOutput",
    "TestSuiteRunExecutionSearchResultsOutput",
    "TestSuiteRunExecutionStringOutput",
    "TestSuiteRunExternalExecConfig",
    "TestSuiteRunExternalExecConfigData",
    "TestSuiteRunExternalExecConfigDataRequest",
    "TestSuiteRunExternalExecConfigRequest",
    "TestSuiteRunMetricArrayOutput",
    "TestSuiteRunMetricErrorOutput",
    "TestSuiteRunMetricJsonOutput",
    "TestSuiteRunMetricNumberOutput",
    "TestSuiteRunMetricOutput",
    "TestSuiteRunMetricStringOutput",
    "TestSuiteRunProgress",
    "TestSuiteRunPromptSandboxExecConfigDataRequest",
    "TestSuiteRunPromptSandboxExecConfigRequest",
    "TestSuiteRunPromptSandboxHistoryItemExecConfig",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigData",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigDataRequest",
    "TestSuiteRunPromptSandboxHistoryItemExecConfigRequest",
    "TestSuiteRunRead",
    "TestSuiteRunState",
    "TestSuiteRunTestSuite",
    "TestSuiteRunWorkflowReleaseTagExecConfig",
    "TestSuiteRunWorkflowReleaseTagExecConfigData",
    "TestSuiteRunWorkflowReleaseTagExecConfigDataRequest",
    "TestSuiteRunWorkflowReleaseTagExecConfigRequest",
    "TestSuiteRunWorkflowSandboxExecConfigDataRequest",
    "TestSuiteRunWorkflowSandboxExecConfigRequest",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfig",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigData",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigDataRequest",
    "TestSuiteRunWorkflowSandboxHistoryItemExecConfigRequest",
    "TestSuiteTestCase",
    "TestSuiteTestCaseBulkOperationRequest",
    "TestSuiteTestCaseBulkResult",
    "TestSuiteTestCaseCreateBulkOperationRequest",
    "TestSuiteTestCaseCreatedBulkResult",
    "TestSuiteTestCaseCreatedBulkResultData",
    "TestSuiteTestCaseDeleteBulkOperationDataRequest",
    "TestSuiteTestCaseDeleteBulkOperationRequest",
    "TestSuiteTestCaseDeletedBulkResult",
    "TestSuiteTestCaseDeletedBulkResultData",
    "TestSuiteTestCaseRejectedBulkResult",
    "TestSuiteTestCaseReplaceBulkOperationRequest",
    "TestSuiteTestCaseReplacedBulkResult",
    "TestSuiteTestCaseReplacedBulkResultData",
    "TestSuiteTestCaseUpsertBulkOperationRequest",
    "ThinkingVellumValue",
    "ThinkingVellumValueRequest",
    "TokenOverlappingWindowChunkerConfig",
    "TokenOverlappingWindowChunkerConfigRequest",
    "TokenOverlappingWindowChunking",
    "TokenOverlappingWindowChunkingRequest",
    "UnitEnum",
    "UploadDocumentResponse",
    "UpsertTestSuiteTestCaseRequest",
    "VariablePromptBlock",
    "VellumAudio",
    "VellumAudioRequest",
    "VellumCodeResourceDefinition",
    "VellumDocument",
    "VellumDocumentRequest",
    "VellumError",
    "VellumErrorCodeEnum",
    "VellumErrorRequest",
    "VellumImage",
    "VellumImageRequest",
    "VellumNodeExecutionEvent",
    "VellumSdkError",
    "VellumSdkErrorCodeEnum",
    "VellumSecret",
    "VellumSpan",
    "VellumValue",
    "VellumValueLogicalConditionGroupRequest",
    "VellumValueLogicalConditionRequest",
    "VellumValueLogicalExpressionRequest",
    "VellumValueRequest",
    "VellumVariable",
    "VellumVariableExtensions",
    "VellumVariableType",
    "VellumVideo",
    "VellumVideoRequest",
    "VellumWorkflowExecutionEvent",
    "VideoChatMessageContent",
    "VideoChatMessageContentRequest",
    "VideoPromptBlock",
    "VideoVellumValue",
    "VideoVellumValueRequest",
    "WorkflowDeploymentEventExecutionsResponse",
    "WorkflowDeploymentHistoryItem",
    "WorkflowDeploymentParentContext",
    "WorkflowDeploymentRead",
    "WorkflowDeploymentRelease",
    "WorkflowDeploymentReleaseWorkflowDeployment",
    "WorkflowDeploymentReleaseWorkflowVersion",
    "WorkflowError",
    "WorkflowEvent",
    "WorkflowEventError",
    "WorkflowEventExecutionRead",
    "WorkflowExecutionActual",
    "WorkflowExecutionActualChatHistoryRequest",
    "WorkflowExecutionActualJsonRequest",
    "WorkflowExecutionActualStringRequest",
    "WorkflowExecutionDetail",
    "WorkflowExecutionEventErrorCode",
    "WorkflowExecutionEventType",
    "WorkflowExecutionFulfilledBody",
    "WorkflowExecutionFulfilledEvent",
    "WorkflowExecutionInitiatedBody",
    "WorkflowExecutionInitiatedEvent",
    "WorkflowExecutionNodeResultEvent",
    "WorkflowExecutionPausedBody",
    "WorkflowExecutionPausedEvent",
    "WorkflowExecutionRejectedBody",
    "WorkflowExecutionRejectedEvent",
    "WorkflowExecutionResumedBody",
    "WorkflowExecutionResumedEvent",
    "WorkflowExecutionSnapshottedBody",
    "WorkflowExecutionSnapshottedEvent",
    "WorkflowExecutionSpan",
    "WorkflowExecutionSpanAttributes",
    "WorkflowExecutionStreamingBody",
    "WorkflowExecutionStreamingEvent",
    "WorkflowExecutionUsageCalculationError",
    "WorkflowExecutionUsageCalculationErrorCodeEnum",
    "WorkflowExecutionUsageCalculationFulfilledBody",
    "WorkflowExecutionUsageResult",
    "WorkflowExecutionViewOnlineEvalMetricResult",
    "WorkflowExecutionWorkflowResultEvent",
    "WorkflowExpandMetaRequest",
    "WorkflowInitializationError",
    "WorkflowNodeResultData",
    "WorkflowNodeResultEvent",
    "WorkflowNodeResultEventState",
    "WorkflowOutput",
    "WorkflowOutputArray",
    "WorkflowOutputChatHistory",
    "WorkflowOutputError",
    "WorkflowOutputFunctionCall",
    "WorkflowOutputImage",
    "WorkflowOutputJson",
    "WorkflowOutputNumber",
    "WorkflowOutputSearchResults",
    "WorkflowOutputString",
    "WorkflowParentContext",
    "WorkflowPushDeploymentConfigRequest",
    "WorkflowPushExecConfig",
    "WorkflowPushResponse",
    "WorkflowReleaseTagRead",
    "WorkflowReleaseTagWorkflowDeploymentHistoryItem",
    "WorkflowRequestChatHistoryInputRequest",
    "WorkflowRequestInputRequest",
    "WorkflowRequestJsonInputRequest",
    "WorkflowRequestNumberInputRequest",
    "WorkflowRequestStringInputRequest",
    "WorkflowResultEvent",
    "WorkflowResultEventOutputData",
    "WorkflowResultEventOutputDataArray",
    "WorkflowResultEventOutputDataChatHistory",
    "WorkflowResultEventOutputDataError",
    "WorkflowResultEventOutputDataFunctionCall",
    "WorkflowResultEventOutputDataJson",
    "WorkflowResultEventOutputDataNumber",
    "WorkflowResultEventOutputDataSearchResults",
    "WorkflowResultEventOutputDataString",
    "WorkflowSandboxExample",
    "WorkflowSandboxParentContext",
    "WorkflowStreamEvent",
    "WorkspaceRead",
    "WorkspaceSecretRead",
]



from typing import Any, Generic, Protocol, TypeVar, Union, runtime_checkable
from typing_extensions import TypeGuard

from vellum.workflows.descriptors.base import BaseDescriptor
from vellum.workflows.descriptors.exceptions import InvalidExpressionException
from vellum.workflows.descriptors.utils import resolve_value
from vellum.workflows.state.base import BaseState

LHS = TypeVar("LHS")
RHS = TypeVar("RHS")


@runtime_checkable
class SupportsLessThan(Protocol):
    def __lt__(self, other: Any) -> bool: ...


def has_lt(obj: Any) -> TypeGuard[SupportsLessThan]:
    return hasattr(obj, "__lt__")


class LessThanExpression(BaseDescriptor[bool], Generic[LHS, RHS]):
    def __init__(
        self,
        *,
        lhs: Union[BaseDescriptor[LHS], LHS],
        rhs: Union[BaseDescriptor[RHS], RHS],
    ) -> None:
        super().__init__(name=f"{lhs} < {rhs}", types=(bool,))
        self._lhs = lhs
        self._rhs = rhs

    def resolve(self, state: "BaseState") -> bool:
        lhs = resolve_value(self._lhs, state)
        rhs = resolve_value(self._rhs, state)

        if not has_lt(lhs):
            raise InvalidExpressionException(f"'{lhs.__class__.__name__}' must support the '<' operator")

        return lhs < rhs



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.paginated_test_suite_run_execution_list import PaginatedTestSuiteRunExecutionList
from ...types.test_suite_run_exec_config_request import TestSuiteRunExecConfigRequest
from ...types.test_suite_run_read import TestSuiteRunRead
from .raw_client import AsyncRawTestSuiteRunsClient, RawTestSuiteRunsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TestSuiteRunsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTestSuiteRunsClient(client_wrapper=client_wrapper)

    @property
    def _client_wrapper(self) -> SyncClientWrapper:
        return self._raw_client._client_wrapper

    @property
    def with_raw_response(self) -> RawTestSuiteRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTestSuiteRunsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        exec_config: TestSuiteRunExecConfigRequest,
        test_suite_id: typing.Optional[str] = OMIT,
        test_suite_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TestSuiteRunRead:
        """
        Trigger a Test Suite and create a new Test Suite Run

        Parameters
        ----------
        exec_config : TestSuiteRunExecConfigRequest
            Configuration that defines how the Test Suite should be run

        test_suite_id : typing.Optional[str]
            The ID of the Test Suite to run. Must provide either this or test_suite_id.

        test_suite_name : typing.Optional[str]
            The name of the Test Suite to run. Must provide either this or test_suite_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestSuiteRunRead


        Examples
        --------
        from vellum import (
            TestSuiteRunDeploymentReleaseTagExecConfigDataRequest,
            TestSuiteRunDeploymentReleaseTagExecConfigRequest,
            Vellum,
        )

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.test_suite_runs.create(
            exec_config=TestSuiteRunDeploymentReleaseTagExecConfigRequest(
                data=TestSuiteRunDeploymentReleaseTagExecConfigDataRequest(
                    deployment_id="deployment_id",
                ),
            ),
        )
        """
        _response = self._raw_client.create(
            exec_config=exec_config,
            test_suite_id=test_suite_id,
            test_suite_name=test_suite_name,
            request_options=request_options,
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> TestSuiteRunRead:
        """
        Retrieve a specific Test Suite Run by ID

        Parameters
        ----------
        id : str
            A UUID string identifying this test suite run.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestSuiteRunRead


        Examples
        --------
        from vellum import Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.test_suite_runs.retrieve(
            id="id",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def list_executions(
        self,
        id: str,
        *,
        expand: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedTestSuiteRunExecutionList:
        """
        Parameters
        ----------
        id : str
            A UUID string identifying this test suite run.

        expand : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The response fields to expand for more information.
            - 'results.metric_results.metric_label' expands the metric label for each metric result.
            - 'results.metric_results.metric_definition' expands the metric definition for each metric result.
            - 'results.metric_results.metric_definition.name' expands the metric definition name for each metric result.

        limit : typing.Optional[int]
            Number of results to return per page.

        offset : typing.Optional[int]
            The initial index from which to return the results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedTestSuiteRunExecutionList


        Examples
        --------
        from vellum import Vellum

        client = Vellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.test_suite_runs.list_executions(
            id="id",
        )
        """
        _response = self._raw_client.list_executions(
            id, expand=expand, limit=limit, offset=offset, request_options=request_options
        )
        return _response.data


class AsyncTestSuiteRunsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTestSuiteRunsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTestSuiteRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTestSuiteRunsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        exec_config: TestSuiteRunExecConfigRequest,
        test_suite_id: typing.Optional[str] = OMIT,
        test_suite_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TestSuiteRunRead:
        """
        Trigger a Test Suite and create a new Test Suite Run

        Parameters
        ----------
        exec_config : TestSuiteRunExecConfigRequest
            Configuration that defines how the Test Suite should be run

        test_suite_id : typing.Optional[str]
            The ID of the Test Suite to run. Must provide either this or test_suite_id.

        test_suite_name : typing.Optional[str]
            The name of the Test Suite to run. Must provide either this or test_suite_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestSuiteRunRead


        Examples
        --------
        import asyncio

        from vellum import (
            AsyncVellum,
            TestSuiteRunDeploymentReleaseTagExecConfigDataRequest,
            TestSuiteRunDeploymentReleaseTagExecConfigRequest,
        )

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.test_suite_runs.create(
                exec_config=TestSuiteRunDeploymentReleaseTagExecConfigRequest(
                    data=TestSuiteRunDeploymentReleaseTagExecConfigDataRequest(
                        deployment_id="deployment_id",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            exec_config=exec_config,
            test_suite_id=test_suite_id,
            test_suite_name=test_suite_name,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> TestSuiteRunRead:
        """
        Retrieve a specific Test Suite Run by ID

        Parameters
        ----------
        id : str
            A UUID string identifying this test suite run.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestSuiteRunRead


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.test_suite_runs.retrieve(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def list_executions(
        self,
        id: str,
        *,
        expand: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedTestSuiteRunExecutionList:
        """
        Parameters
        ----------
        id : str
            A UUID string identifying this test suite run.

        expand : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The response fields to expand for more information.
            - 'results.metric_results.metric_label' expands the metric label for each metric result.
            - 'results.metric_results.metric_definition' expands the metric definition for each metric result.
            - 'results.metric_results.metric_definition.name' expands the metric definition name for each metric result.

        limit : typing.Optional[int]
            Number of results to return per page.

        offset : typing.Optional[int]
            The initial index from which to return the results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedTestSuiteRunExecutionList


        Examples
        --------
        import asyncio

        from vellum import AsyncVellum

        client = AsyncVellum(
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.test_suite_runs.list_executions(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_executions(
            id, expand=expand, limit=limit, offset=offset, request_options=request_options
        )
        return _response.data



from uuid import UUID
from typing import Generic, Optional, TypeVar

from vellum.workflows.nodes.core.templating_node import TemplatingNode
from vellum.workflows.types.core import JsonObject
from vellum.workflows.utils.vellum_variables import primitive_type_to_vellum_variable_type
from vellum_ee.workflows.display.nodes.base_node_display import BaseNodeDisplay
from vellum_ee.workflows.display.nodes.utils import raise_if_descriptor
from vellum_ee.workflows.display.nodes.vellum.utils import create_node_input
from vellum_ee.workflows.display.types import WorkflowDisplayContext

_TemplatingNodeType = TypeVar("_TemplatingNodeType", bound=TemplatingNode)

TEMPLATE_INPUT_NAME = TemplatingNode.template.name


class BaseTemplatingNodeDisplay(BaseNodeDisplay[_TemplatingNodeType], Generic[_TemplatingNodeType]):
    __serializable_inputs__ = {TemplatingNode.inputs}

    def serialize(
        self, display_context: WorkflowDisplayContext, error_output_id: Optional[UUID] = None, **_kwargs
    ) -> JsonObject:
        node = self._node
        node_id = self.node_id

        template_input_id = self.node_input_ids_by_name.get(TEMPLATE_INPUT_NAME)

        template_node_input = create_node_input(
            node_id=node_id,
            input_name=TEMPLATE_INPUT_NAME,
            value=node.template,
            display_context=display_context,
            input_id=template_input_id,
        )
        template_node_inputs = raise_if_descriptor(node.inputs)
        template_inputs = [
            create_node_input(
                node_id=node_id,
                input_name=variable_name,
                value=variable_value,
                display_context=display_context,
                input_id=self.node_input_ids_by_name.get(f"{TemplatingNode.inputs.name}.{variable_name}")
                or self.node_input_ids_by_name.get(variable_name),
            )
            for variable_name, variable_value in template_node_inputs.items()
            if variable_name != TEMPLATE_INPUT_NAME
        ]
        node_inputs = [template_node_input, *template_inputs]

        # Misc type ignore is due to `node.Outputs` being generic
        # https://app.shortcut.com/vellum/story/4784
        output_descriptor = node.Outputs.result  # type: ignore [misc]
        _, output_display = display_context.global_node_output_displays[output_descriptor]
        inferred_output_type = primitive_type_to_vellum_variable_type(output_descriptor)

        return {
            "id": str(node_id),
            "type": "TEMPLATING",
            "inputs": [node_input.dict() for node_input in node_inputs],
            "data": {
                "label": self.label,
                "output_id": str(output_display.id),
                "error_output_id": str(error_output_id) if error_output_id else None,
                "source_handle_id": str(self.get_source_handle_id(display_context.port_displays)),
                "target_handle_id": str(self.get_target_handle_id()),
                "template_node_input_id": str(template_node_input.id),
                "output_type": inferred_output_type,
            },
            **self.serialize_generic_fields(display_context),
        }



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .vellum_variable import VellumVariable


class MetricDefinitionHistoryItem(UniversalBaseModel):
    id: str
    label: str = pydantic.Field()
    """
    A human-readable label for the metric
    """

    name: str = pydantic.Field()
    """
    A name that uniquely identifies this metric within its workspace
    """

    description: str
    input_variables: typing.List[VellumVariable]
    output_variables: typing.List[VellumVariable]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .array_vellum_value import ArrayVellumValue  # noqa: E402, F401, I001

update_forward_refs(MetricDefinitionHistoryItem)



from vellum.workflows.constants import APIRequestMethod
from vellum.workflows.nodes.displayable.api_node.node import APINode
from vellum.workflows.workflows.base import BaseWorkflow
from vellum_ee.workflows.display.workflows.get_vellum_workflow_display_class import get_workflow_display


def test_serialize_node__api_node_with_timeout():
    # GIVEN an API node with timeout specified
    class MyAPINode(APINode):
        url = "https://api.example.com"
        method = APIRequestMethod.GET
        timeout = 30  # This is the key attribute we're testing

    # AND a workflow with the API node
    class Workflow(BaseWorkflow):
        graph = MyAPINode

    # WHEN the workflow is serialized
    workflow_display = get_workflow_display(workflow_class=Workflow)
    serialized_workflow: dict = workflow_display.serialize()

    # THEN the node should properly serialize the timeout
    my_api_node = next(node for node in serialized_workflow["workflow_raw_data"]["nodes"] if node["type"] == "API")

    # Assert that timeout is present in the serialized attributes
    timeout_attribute = next((attr for attr in my_api_node.get("attributes", []) if attr["name"] == "timeout"), None)

    assert timeout_attribute is not None, "timeout attribute should be present in serialized attributes"
    assert timeout_attribute["value"]["type"] == "CONSTANT_VALUE"
    assert timeout_attribute["value"]["value"]["type"] == "NUMBER"
    assert timeout_attribute["value"]["value"]["value"] == 30.0



# This file was auto-generated by Fern from our API Definition.

import typing

from .fulfilled_ad_hoc_execute_prompt_event import FulfilledAdHocExecutePromptEvent
from .initiated_ad_hoc_execute_prompt_event import InitiatedAdHocExecutePromptEvent
from .rejected_ad_hoc_execute_prompt_event import RejectedAdHocExecutePromptEvent
from .streaming_ad_hoc_execute_prompt_event import StreamingAdHocExecutePromptEvent

AdHocExecutePromptEvent = typing.Union[
    InitiatedAdHocExecutePromptEvent,
    StreamingAdHocExecutePromptEvent,
    FulfilledAdHocExecutePromptEvent,
    RejectedAdHocExecutePromptEvent,
]



from typing import TYPE_CHECKING, TypeVar, cast

from vellum.workflows.descriptors.base import BaseDescriptor
from vellum.workflows.errors.types import WorkflowErrorCode
from vellum.workflows.exceptions import NodeException

if TYPE_CHECKING:
    from vellum.workflows.state.base import BaseState


_T = TypeVar("_T")


class StateValueReference(BaseDescriptor[_T]):

    def resolve(self, state: "BaseState") -> _T:
        if hasattr(state, self._name):
            return cast(_T, getattr(state, self._name))

        if state.meta.parent:
            return self.resolve(state.meta.parent)

        raise NodeException(f"Missing required Workflow state: {self._name}", code=WorkflowErrorCode.INVALID_STATE)



import pytest
from dataclasses import dataclass
import json
from uuid import uuid4
from typing import Any, Iterator, List

from httpx import Response

from vellum.client.core.api_error import ApiError
from vellum.client.core.pydantic_utilities import UniversalBaseModel
from vellum.client.types.chat_message import ChatMessage
from vellum.client.types.chat_message_prompt_block import ChatMessagePromptBlock
from vellum.client.types.chat_message_request import ChatMessageRequest
from vellum.client.types.execute_prompt_event import ExecutePromptEvent
from vellum.client.types.fulfilled_execute_prompt_event import FulfilledExecutePromptEvent
from vellum.client.types.function_call import FunctionCall
from vellum.client.types.function_call_vellum_value import FunctionCallVellumValue
from vellum.client.types.function_definition import FunctionDefinition
from vellum.client.types.initiated_execute_prompt_event import InitiatedExecutePromptEvent
from vellum.client.types.prompt_output import PromptOutput
from vellum.client.types.prompt_request_chat_history_input import PromptRequestChatHistoryInput
from vellum.client.types.prompt_request_json_input import PromptRequestJsonInput
from vellum.client.types.string_vellum_value import StringVellumValue
from vellum.workflows.context import execution_context
from vellum.workflows.errors.types import WorkflowErrorCode
from vellum.workflows.exceptions import NodeException
from vellum.workflows.nodes.displayable.inline_prompt_node.node import InlinePromptNode


def test_inline_prompt_node__json_inputs(vellum_adhoc_prompt_client):
    # GIVEN a prompt node with various inputs
    @dataclass
    class MyDataClass:
        hello: str

    class MyPydantic(UniversalBaseModel):
        example: str

    class MyNode(InlinePromptNode):
        ml_model = "gpt-4o"
        blocks = []
        prompt_inputs = {
            "a_dict": {"foo": "bar"},
            "a_list": [1, 2, 3],
            "a_dataclass": MyDataClass(hello="world"),
            "a_pydantic": MyPydantic(example="example"),
            "an_empty_list": [],
        }

    # AND a known response from invoking an inline prompt
    expected_outputs: List[PromptOutput] = [
        StringVellumValue(value="Test"),
    ]

    def generate_prompt_events(*args: Any, **kwargs: Any) -> Iterator[ExecutePromptEvent]:
        execution_id = str(uuid4())
        events: List[ExecutePromptEvent] = [
            InitiatedExecutePromptEvent(execution_id=execution_id),
            FulfilledExecutePromptEvent(
                execution_id=execution_id,
                outputs=expected_outputs,
            ),
        ]
        yield from events

    vellum_adhoc_prompt_client.adhoc_execute_prompt_stream.side_effect = generate_prompt_events

    # WHEN the node is run
    list(MyNode().run())

    # THEN the prompt is executed with the correct inputs
    mock_api = vellum_adhoc_prompt_client.adhoc_execute_prompt_stream
    assert mock_api.call_count == 1
    assert mock_api.call_args.kwargs["input_values"] == [
        PromptRequestJsonInput(key="a_dict", type="JSON", value={"foo": "bar"}),
        PromptRequestJsonInput(key="a_list", type="JSON", value=[1, 2, 3]),
        PromptRequestJsonInput(key="a_dataclass", type="JSON", value={"hello": "world"}),
        PromptRequestJsonInput(key="a_pydantic", type="JSON", value={"example": "example"}),
        PromptRequestJsonInput(key="an_empty_list", type="JSON", value=[]),
    ]
    assert len(mock_api.call_args.kwargs["input_variables"]) == 5


def test_inline_prompt_node__function_definitions(vellum_adhoc_prompt_client):
    # GIVEN a function definition
    def my_function(foo: str, bar: int) -> None:
        pass

    # AND a prompt node with a accepting that function definition
    class MyNode(InlinePromptNode):
        ml_model = "gpt-4o"
        functions = [my_function]
        blocks = []

    # AND a known response from invoking an inline prompt
    expected_outputs: List[PromptOutput] = [
        FunctionCallVellumValue(value=FunctionCall(name="my_function", arguments={"foo": "hello", "bar": 1})),
    ]

    def generate_prompt_events(*args: Any, **kwargs: Any) -> Iterator[ExecutePromptEvent]:
        execution_id = str(uuid4())
        events: List[ExecutePromptEvent] = [
            InitiatedExecutePromptEvent(execution_id=execution_id),
            FulfilledExecutePromptEvent(
                execution_id=execution_id,
                outputs=expected_outputs,
            ),
        ]
        yield from events

    vellum_adhoc_prompt_client.adhoc_execute_prompt_stream.side_effect = generate_prompt_events

    # WHEN the node is run
    outputs = list(MyNode().run())

    # THEN the prompt is executed with the correct inputs
    mock_api = vellum_adhoc_prompt_client.adhoc_execute_prompt_stream
    assert mock_api.call_count == 1
    assert mock_api.call_args.kwargs["functions"] == [
        FunctionDefinition(
            name="my_function",
            parameters={
                "type": "object",
                "properties": {
                    "foo": {"type": "string"},
                    "bar": {"type": "integer"},
                },
                "required": ["foo", "bar"],
            },
        ),
    ]
    assert (
        outputs[-1].value
        == """{
    "arguments": {
        "foo": "hello",
        "bar": 1
    },
    "id": null,
    "name": "my_function"
}"""
    )


@pytest.mark.parametrize(
    ["exception", "expected_code", "expected_message"],
    [
        (
            ApiError(status_code=404, body={"detail": "Model not found"}),
            WorkflowErrorCode.INVALID_INPUTS,
            "Model not found",
        ),
        (
            ApiError(status_code=404, body={"message": "Model not found"}),
            WorkflowErrorCode.INVALID_INPUTS,
            "Failed to execute Prompt",
        ),
        (
            ApiError(status_code=404, body="Model not found"),
            WorkflowErrorCode.INTERNAL_ERROR,
            "Failed to execute Prompt",
        ),
        (
            ApiError(status_code=None, body={"detail": "Model not found"}),
            WorkflowErrorCode.INTERNAL_ERROR,
            "Failed to execute Prompt",
        ),
        (
            ApiError(status_code=500, body={"detail": "Model not found"}),
            WorkflowErrorCode.INTERNAL_ERROR,
            "Failed to execute Prompt",
        ),
        (
            ApiError(status_code=403, body={"detail": "Provider credentials is missing or unavailable"}),
            WorkflowErrorCode.PROVIDER_CREDENTIALS_UNAVAILABLE,
            "Provider credentials is missing or unavailable",
        ),
    ],
    ids=["404", "invalid_dict", "invalid_body", "no_status_code", "500", "403"],
)
def test_inline_prompt_node__api_error__invalid_inputs_node_exception(
    vellum_adhoc_prompt_client, exception, expected_code, expected_message
):
    # GIVEN a prompt node with an invalid model name
    class MyNode(InlinePromptNode):
        ml_model = "my-invalid-model"
        blocks = []

    # AND the adhoc prompt client raises a 4xx error
    vellum_adhoc_prompt_client.adhoc_execute_prompt_stream.side_effect = exception

    # WHEN the node is run
    with pytest.raises(NodeException) as e:
        list(MyNode().run())

    # THEN the node raises the correct NodeException
    assert e.value.code == expected_code
    assert e.value.message == expected_message


def test_inline_prompt_node__chat_history_inputs(vellum_adhoc_prompt_client):
    # GIVEN a prompt node with a chat history input
    class MyNode(InlinePromptNode):
        ml_model = "gpt-4o"
        blocks = []
        prompt_inputs = {
            "chat_history": [ChatMessageRequest(role="USER", text="Hello, how are you?")],
        }

    # AND a known response from invoking an inline prompt
    expected_outputs: List[PromptOutput] = [
        StringVellumValue(value="Great!"),
    ]

    def generate_prompt_events(*args: Any, **kwargs: Any) -> Iterator[ExecutePromptEvent]:
        execution_id = str(uuid4())
        events: List[ExecutePromptEvent] = [
            InitiatedExecutePromptEvent(execution_id=execution_id),
            FulfilledExecutePromptEvent(
                execution_id=execution_id,
                outputs=expected_outputs,
            ),
        ]
        yield from events

    vellum_adhoc_prompt_client.adhoc_execute_prompt_stream.side_effect = generate_prompt_events

    # WHEN the node is run
    events = list(MyNode().run())

    # THEN the prompt is executed with the correct inputs
    assert events[-1].value == "Great!"

    # AND the prompt is executed with the correct inputs
    mock_api = vellum_adhoc_prompt_client.adhoc_execute_prompt_stream
    assert mock_api.call_count == 1
    assert mock_api.call_args.kwargs["input_values"] == [
        PromptRequestChatHistoryInput(
            key="chat_history",
            type="CHAT_HISTORY",
            value=[ChatMessage(role="USER", text="Hello, how are you?")],
        ),
    ]
    assert mock_api.call_args.kwargs["input_variables"][0].type == "CHAT_HISTORY"


@pytest.mark.timeout(5)
def test_inline_prompt_node__parent_context(mock_httpx_transport, mock_complex_parent_context):
    # GIVEN a prompt node
    class MyNode(InlinePromptNode):
        ml_model = "gpt-4o"
        blocks = [ChatMessagePromptBlock(chat_role="USER", blocks=[])]
        prompt_inputs = {}

    # AND a known response from the httpx client
    expected_outputs: List[PromptOutput] = [
        StringVellumValue(value="Test"),
    ]
    execution_id = str(uuid4())
    events: List[ExecutePromptEvent] = [
        InitiatedExecutePromptEvent(execution_id=execution_id),
        FulfilledExecutePromptEvent(
            execution_id=execution_id,
            outputs=expected_outputs,
        ),
    ]
    text = "\n".join(e.model_dump_json() for e in events)

    mock_httpx_transport.handle_request.return_value = Response(
        status_code=200,
        text=text,
    )

    # WHEN the node is run with the complex parent context
    trace_id = uuid4()
    with execution_context(
        parent_context=mock_complex_parent_context,
        trace_id=trace_id,
    ):
        outputs = list(MyNode().run())

    # THEN the last output is as expected
    assert outputs[-1].value == "Test"

    # AND the prompt is executed with the correct execution context
    call_request_args = mock_httpx_transport.handle_request.call_args_list[0][0][0]
    call_request = json.loads(call_request_args.read().decode("utf-8"))
    request_execution_context = call_request["execution_context"]
    assert request_execution_context["trace_id"] == str(trace_id)
    assert request_execution_context["parent_context"]

    # AND the blocks are serialized as expected
    assert call_request["blocks"] == [
        {
            "block_type": "CHAT_MESSAGE",
            "chat_role": "USER",
            "blocks": [],
        }
    ]



import pytest
from uuid import UUID
from typing import Type

from vellum.client.core.api_error import ApiError
from vellum.workflows.nodes.displayable.code_execution_node.node import CodeExecutionNode
from vellum.workflows.references.vellum_secret import VellumSecretReference
from vellum.workflows.workflows.base import BaseWorkflow
from vellum_ee.workflows.display.exceptions import NodeValidationError
from vellum_ee.workflows.display.nodes.vellum.code_execution_node import BaseCodeExecutionNodeDisplay
from vellum_ee.workflows.display.workflows.get_vellum_workflow_display_class import get_workflow_display


def _no_display_class(Node: Type[CodeExecutionNode]):
    return None


def _display_class_with_node_input_ids_by_name(Node: Type[CodeExecutionNode]):
    class CodeExecutionNodeDisplay(BaseCodeExecutionNodeDisplay[Node]):  # type: ignore[valid-type]
        node_input_ids_by_name = {"foo": UUID("fba6a4d5-835a-4e99-afb7-f6a4aed15110")}

    return CodeExecutionNodeDisplay


def _display_class_with_node_input_ids_by_name_with_inputs_prefix(Node: Type[CodeExecutionNode]):
    class CodeExecutionNodeDisplay(BaseCodeExecutionNodeDisplay[Node]):  # type: ignore[valid-type]
        node_input_ids_by_name = {"code_inputs.foo": UUID("fba6a4d5-835a-4e99-afb7-f6a4aed15110")}

    return CodeExecutionNodeDisplay


@pytest.mark.parametrize(
    ["GetDisplayClass", "expected_input_id"],
    [
        (_no_display_class, "a5dbe403-0b00-4df6-b8f7-ed5f7794b003"),
        (_display_class_with_node_input_ids_by_name, "fba6a4d5-835a-4e99-afb7-f6a4aed15110"),
        (_display_class_with_node_input_ids_by_name_with_inputs_prefix, "fba6a4d5-835a-4e99-afb7-f6a4aed15110"),
    ],
    ids=[
        "no_display_class",
        "display_class_with_node_input_ids_by_name",
        "display_class_with_node_input_ids_by_name_with_inputs_prefix",
    ],
)
def test_serialize_node__code_node_inputs(GetDisplayClass, expected_input_id):
    # GIVEN a code node with inputs
    class MyCodeExecutionNode(CodeExecutionNode):
        code_inputs = {"foo": "bar"}

    # AND a workflow with the code node
    class Workflow(BaseWorkflow):
        graph = MyCodeExecutionNode

    # AND a display class
    GetDisplayClass(MyCodeExecutionNode)

    # WHEN the workflow is serialized
    workflow_display = get_workflow_display(workflow_class=Workflow)
    serialized_workflow: dict = workflow_display.serialize()

    # THEN the node should properly serialize the inputs
    my_code_execution_node = next(
        node for node in serialized_workflow["workflow_raw_data"]["nodes"] if node["type"] == "CODE_EXECUTION"
    )

    assert my_code_execution_node["inputs"] == [
        {
            "id": expected_input_id,
            "key": "foo",
            "value": {
                "combinator": "OR",
                "rules": [
                    {
                        "type": "CONSTANT_VALUE",
                        "data": {
                            "type": "STRING",
                            "value": "bar",
                        },
                    }
                ],
            },
        },
        {
            "id": "9774d864-c76d-4a1a-8181-b632ed3ab87c",
            "key": "code",
            "value": {
                "combinator": "OR",
                "rules": [
                    {
                        "type": "CONSTANT_VALUE",
                        "data": {
                            "type": "STRING",
                            "value": "",
                        },
                    }
                ],
            },
        },
        {
            "id": "34742235-5699-45cd-9d34-bce3745e743d",
            "key": "runtime",
            "value": {
                "combinator": "OR",
                "rules": [
                    {
                        "type": "CONSTANT_VALUE",
                        "data": {
                            "type": "STRING",
                            "value": "PYTHON_3_11_6",
                        },
                    }
                ],
            },
        },
    ]


def test_serialize_node__with_unresolved_secret_references(vellum_client):
    # GIVEN a node has access to a secret reference
    class MyNode(CodeExecutionNode):
        code_inputs = {"api_key": VellumSecretReference("MY_API_KEY")}

    # AND the secret is not found
    vellum_client.workspace_secrets.retrieve.side_effect = ApiError(
        status_code=404,
        body="Secret not found",
    )

    # AND a workflow with the code node
    class Workflow(BaseWorkflow):
        graph = MyNode

    # WHEN we serialize the workflow
    workflow_display = get_workflow_display(workflow_class=Workflow)
    data: dict = workflow_display.serialize()

    # THEN the condition should be serialized correctly
    node = next(node for node in data["workflow_raw_data"]["nodes"] if node["type"] == "CODE_EXECUTION")
    assert node["inputs"][0]["value"] == {
        "combinator": "OR",
        "rules": [
            {
                "type": "WORKSPACE_SECRET",
                "data": {
                    "type": "STRING",
                    "workspace_secret_id": None,
                },
            }
        ],
    }

    # AND we should have a warning of the invalid reference
    # TODO: Come up with a proposal for how nodes should propagate warnings
    # warnings = list(workflow_display.errors)
    # assert len(warnings) == 1
    # assert "Failed to resolve secret reference 'MY_API_KEY'" in str(warnings[0])


def test_serialize_node__with_non_exist_code_input_path():
    # GIVEN a code node with a non-existent code input path
    class MyNode(CodeExecutionNode):
        filepath = "non_existent_file.py"

    # AND a workflow with the code node
    class Workflow(BaseWorkflow):
        graph = MyNode

    # WHEN we serialize the workflow
    workflow_display = get_workflow_display(workflow_class=Workflow)
    with pytest.raises(NodeValidationError) as exc_info:
        workflow_display.serialize()
    assert "Filepath 'non_existent_file.py' does not exist" in str(exc_info.value)


def test_serialize_node__with_non_exist_code_input_path_with_dry_run():
    # GIVEN a code node with a non-existent code input path
    class MyNode(CodeExecutionNode):
        filepath = "non_existent_file.py"

    # AND a workflow with the code node
    class Workflow(BaseWorkflow):
        graph = MyNode

    # WHEN we serialize the workflow with dry_run=True
    workflow_display = get_workflow_display(workflow_class=Workflow, dry_run=True)
    data: dict = workflow_display.serialize()

    # THEN the workflow should not raise an error
    assert data == {
        "workflow_raw_data": {
            "nodes": [
                {
                    "id": "9b9e2a5d-01a4-46b2-80a3-d9484b2c0e08",
                    "type": "ENTRYPOINT",
                    "inputs": [],
                    "data": {"label": "Entrypoint Node", "source_handle_id": "3e2a3f52-5047-4e2e-9a21-37bd43c63250"},
                    "display_data": {"position": {"x": 0.0, "y": -50.0}},
                    "base": None,
                    "definition": None,
                },
                {
                    "id": "ac90c0ce-f393-438c-a24f-e5e9a9286182",
                    "type": "CODE_EXECUTION",
                    "inputs": [
                        {
                            "id": "a0b9d6f6-ce59-4075-8db6-866781bc73ef",
                            "key": "code",
                            "value": {
                                "rules": [{"type": "CONSTANT_VALUE", "data": {"type": "JSON", "value": None}}],
                                "combinator": "OR",
                            },
                        },
                        {
                            "id": "58598cc8-aa8a-4b4f-99fb-09f6815b6c01",
                            "key": "runtime",
                            "value": {
                                "rules": [
                                    {"type": "CONSTANT_VALUE", "data": {"type": "STRING", "value": "PYTHON_3_11_6"}}
                                ],
                                "combinator": "OR",
                            },
                        },
                    ],
                    "data": {
                        "label": "My Node",
                        "error_output_id": None,
                        "source_handle_id": "7afa3858-f50c-4116-936a-a401e3b2c60f",
                        "target_handle_id": "3a39ea63-9f86-4891-a902-0216a7190720",
                        "code_input_id": "a0b9d6f6-ce59-4075-8db6-866781bc73ef",
                        "runtime_input_id": "58598cc8-aa8a-4b4f-99fb-09f6815b6c01",
                        "output_type": "STRING",
                        "packages": [],
                        "output_id": "00b2120e-b642-46e4-8276-5f3c69d8a6cb",
                        "log_output_id": "47e3eeca-4bf8-492e-b8ac-28c7d389c886",
                    },
                    "display_data": {"position": {"x": 200.0, "y": -50.0}},
                    "base": {
                        "name": "CodeExecutionNode",
                        "module": ["vellum", "workflows", "nodes", "displayable", "code_execution_node", "node"],
                    },
                    "definition": {
                        "name": "MyNode",
                        "module": [
                            "vellum_ee",
                            "workflows",
                            "display",
                            "nodes",
                            "vellum",
                            "tests",
                            "test_code_execution_node",
                        ],
                    },
                    "ports": [{"id": "7afa3858-f50c-4116-936a-a401e3b2c60f", "name": "default", "type": "DEFAULT"}],
                    "trigger": {"id": "3a39ea63-9f86-4891-a902-0216a7190720", "merge_behavior": "AWAIT_ANY"},
                },
            ],
            "edges": [
                {
                    "id": "ab6ef06e-df2c-4877-9c3e-9d7261b39748",
                    "source_node_id": "9b9e2a5d-01a4-46b2-80a3-d9484b2c0e08",
                    "source_handle_id": "3e2a3f52-5047-4e2e-9a21-37bd43c63250",
                    "target_node_id": "ac90c0ce-f393-438c-a24f-e5e9a9286182",
                    "target_handle_id": "3a39ea63-9f86-4891-a902-0216a7190720",
                    "type": "DEFAULT",
                }
            ],
            "display_data": {"viewport": {"x": 0.0, "y": 0.0, "zoom": 1.0}},
            "definition": {
                "name": "Workflow",
                "module": ["vellum_ee", "workflows", "display", "nodes", "vellum", "tests", "test_code_execution_node"],
            },
            "output_values": [],
        },
        "input_variables": [],
        "state_variables": [],
        "output_variables": [],
    }



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TestSuiteTestCaseRejectedBulkResult(UniversalBaseModel):
    """
    The result of a bulk operation that failed to operate on a Test Case.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An ID that maps back to one of the initially supplied operations. Can be used to determine the result of a given operation.
    """

    type: typing.Literal["REJECTED"] = "REJECTED"
    data: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Details about the error that occurred
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import pytest
from copy import deepcopy
import json
from queue import Queue
from typing import Dict, List, cast

from vellum.workflows.nodes.bases import BaseNode
from vellum.workflows.outputs.base import BaseOutputs
from vellum.workflows.state.base import BaseState
from vellum.workflows.state.delta import SetStateDelta, StateDelta
from vellum.workflows.state.encoder import DefaultStateEncoder


@pytest.fixture()
def mock_deepcopy(mocker):
    return mocker.patch("vellum.workflows.state.base.deepcopy")


@pytest.fixture
def mock_logger(mocker):
    return mocker.patch("vellum.workflows.state.base.logger")


class MockState(BaseState):
    foo: str
    nested_dict: Dict[str, int] = {}

    __snapshot_count__: int = 0

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.__snapshot_callback__ = lambda _, __: self.__mock_snapshot__()

    def __mock_snapshot__(self) -> None:
        self.__snapshot_count__ += 1

    def __deepcopy__(self, memo: dict) -> "MockState":
        new_state = cast(MockState, super().__deepcopy__(memo))
        new_state.__snapshot_count__ = 0
        return new_state


class MockNode(BaseNode):
    class ExternalInputs(BaseNode.ExternalInputs):
        message: str

    class Outputs(BaseOutputs):
        baz: str


MOCK_NODE_OUTPUT_ID = "e4dc3136-0c27-4bda-b3ab-ea355d5219d6"


def test_state_snapshot__node_attribute_edit():
    # GIVEN an initial state instance
    state = MockState(foo="bar")
    assert state.__snapshot_count__ == 0

    # WHEN we edit an attribute
    state.foo = "baz"

    # THEN the snapshot is emitted
    assert state.__snapshot_count__ == 1


def test_state_snapshot__node_output_edit():
    # GIVEN an initial state instance
    state = MockState(foo="bar")
    assert state.__snapshot_count__ == 0

    # WHEN we add a Node Output to state
    for output in MockNode.Outputs:
        state.meta.node_outputs[output] = "hello"

    # THEN the snapshot is emitted
    assert state.__snapshot_count__ == 1


def test_state_snapshot__nested_dictionary_edit():
    # GIVEN an initial state instance
    state = MockState(foo="bar")
    assert state.__snapshot_count__ == 0

    # WHEN we edit a nested dictionary
    state.nested_dict["hello"] = 1

    # THEN the snapshot is emitted
    assert state.__snapshot_count__ == 1


def test_state_snapshot__external_input_edit():
    # GIVEN an initial state instance
    state = MockState(foo="bar")
    assert state.__snapshot_count__ == 0

    # WHEN we add an external input to state
    state.meta.external_inputs[MockNode.ExternalInputs.message] = "hello"

    # THEN the snapshot is emitted
    assert state.__snapshot_count__ == 1


def test_state_deepcopy():
    # GIVEN an initial state instance
    state = MockState(foo="bar")

    # AND we add a Node Output to state
    state.meta.node_outputs[MockNode.Outputs.baz] = "hello"

    # WHEN we deepcopy the state
    deepcopied_state = deepcopy(state)

    # THEN node outputs are deepcopied
    assert deepcopied_state.meta.node_outputs == state.meta.node_outputs


def test_state_deepcopy__with_node_output_updates():
    # GIVEN an initial state instance
    state = MockState(foo="bar")

    # AND we add a Node Output to state
    state.meta.node_outputs[MockNode.Outputs.baz] = "hello"

    # AND we deepcopy the state
    deepcopied_state = deepcopy(state)

    # AND we update the original state
    state.meta.node_outputs[MockNode.Outputs.baz] = "world"

    # THEN the copied state is not updated
    assert deepcopied_state.meta.node_outputs[MockNode.Outputs.baz] == "hello"

    # AND the original state has had the correct number of snapshots
    assert state.__snapshot_count__ == 2

    # AND the copied state has had the correct number of snapshots
    assert deepcopied_state.__snapshot_count__ == 0


def test_state_json_serialization__with_node_output_updates():
    # GIVEN an initial state instance
    state = MockState(foo="bar")

    # AND we add a Node Output to state
    state.meta.node_outputs[MockNode.Outputs.baz] = "hello"

    # WHEN we serialize the state
    json_state = json.loads(json.dumps(state, cls=DefaultStateEncoder))

    # THEN the state is serialized correctly
    assert json_state["meta"]["node_outputs"] == {MOCK_NODE_OUTPUT_ID: "hello"}


def test_state_deepcopy__with_external_input_updates():
    # GIVEN an initial state instance
    state = MockState(foo="bar")

    # AND we add an external input to state
    state.meta.external_inputs[MockNode.ExternalInputs.message] = "hello"

    # AND we deepcopy the state
    deepcopied_state = deepcopy(state)

    # AND we update the original state
    state.meta.external_inputs[MockNode.ExternalInputs.message] = "world"

    # THEN the copied state is not updated
    assert deepcopied_state.meta.external_inputs[MockNode.ExternalInputs.message] == "hello"

    # AND the original state has had the correct number of snapshots
    assert state.__snapshot_count__ == 2

    # AND the copied state has had the correct number of snapshots
    assert deepcopied_state.__snapshot_count__ == 0


def test_state_json_serialization__with_queue():
    # GIVEN an initial state instance
    state = MockState(foo="bar")

    # AND we add a Node Output queue to state
    queue: Queue[str] = Queue()
    queue.put("test1")
    queue.put("test2")

    state.meta.node_outputs[MockNode.Outputs.baz] = queue

    # WHEN we serialize the state
    json_state = json.loads(json.dumps(state, cls=DefaultStateEncoder))

    # THEN the state is serialized correctly with the queue turned into a list
    assert json_state["meta"]["node_outputs"] == {MOCK_NODE_OUTPUT_ID: ["test1", "test2"]}


def test_state_snapshot__deepcopy_fails__logs_error(mock_deepcopy, mock_logger):
    # GIVEN an initial state instance
    class MockState(BaseState):
        foo: str

    state = MockState(foo="bar")

    # AND we have a snapshot callback that we are tracking
    snapshot_count = 0

    def __snapshot_callback__(state: "BaseState", deltas: List[StateDelta]) -> None:
        nonlocal snapshot_count
        snapshot_count += 1

    state.__snapshot_callback__ = __snapshot_callback__

    # AND deepcopy fails the first time but succeeds the second time
    deepcopy_side_effect_count = 0

    def side_effect(*args, **kwargs):
        nonlocal deepcopy_side_effect_count
        deepcopy_side_effect_count += 1
        if deepcopy_side_effect_count == 1:
            raise Exception("Failed to deepcopy")
        return deepcopy(args[0])

    mock_deepcopy.side_effect = side_effect

    # WHEN we snapshot the state twice
    state.__snapshot__(SetStateDelta(name="foo", delta="bar"))
    state.__snapshot__(SetStateDelta(name="foo", delta="baz"))

    # THEN we were able to invoke the callback once
    assert snapshot_count == 1

    # AND alert sentry once
    assert mock_logger.exception.call_count == 1



import pytest
import os
from unittest.mock import patch

from vellum.client.types.execute_api_response import ExecuteApiResponse
from vellum.workflows.constants import APIRequestMethod
from vellum.workflows.errors.types import WorkflowErrorCode
from vellum.workflows.exceptions import NodeException
from vellum.workflows.nodes.displayable.bases.api_node.node import BaseAPINode
from vellum.workflows.types.core import VellumSecret


@pytest.mark.parametrize("method_value", ["GET", "get", APIRequestMethod.GET])
def test_api_node_with_string_method(method_value, vellum_client):
    class TestAPINode(BaseAPINode):
        method = method_value
        url = "https://example.com"
        headers = {"Authorization": VellumSecret(name="API_KEY")}

    mock_response = ExecuteApiResponse(
        json_={"status": "success"},
        headers={"content-type": "application/json"},
        status_code=200,
        text='{"status": "success"}',
    )
    vellum_client.execute_api.return_value = mock_response

    node = TestAPINode()
    result = node.run()

    assert result.status_code == 200

    vellum_client.execute_api.assert_called_once()
    call_args = vellum_client.execute_api.call_args
    assert call_args[1]["method"] == "GET"


def test_api_node_with_invalid_method():
    class TestAPINode(BaseAPINode):
        method = "INVALID_METHOD"
        url = "https://example.com"

    node = TestAPINode()

    with pytest.raises(NodeException) as exc_info:
        node.run()

    assert exc_info.value.code == WorkflowErrorCode.INVALID_INPUTS
    assert "Invalid HTTP method 'INVALID_METHOD'" == str(exc_info.value)


def test_api_node_adds_user_agent_header_when_none_provided(requests_mock):
    """
    Tests that the API node adds User-Agent header when no headers are provided.
    """

    class TestAPINode(BaseAPINode):
        method = APIRequestMethod.GET
        url = "https://example.com/test"

    response_mock = requests_mock.get(
        "https://example.com/test",
        json={"result": "success"},
        status_code=200,
    )

    node = TestAPINode()
    result = node.run()

    assert response_mock.last_request
    assert "vellum-ai" in response_mock.last_request.headers.get("User-Agent", "")

    assert result.status_code == 200


def test_api_node_adds_user_agent_header_when_headers_provided_without_user_agent(requests_mock):
    """
    Tests that the API node adds User-Agent header when headers are provided but don't include User-Agent.
    """

    class TestAPINode(BaseAPINode):
        method = APIRequestMethod.POST
        url = "https://example.com/test"
        headers = {"Content-Type": "application/json", "Custom-Header": "value"}
        json = {"test": "data"}

    response_mock = requests_mock.post(
        "https://example.com/test",
        json={"result": "success"},
        status_code=200,
    )

    node = TestAPINode()
    result = node.run()

    assert response_mock.last_request
    assert "vellum-ai" in response_mock.last_request.headers.get("User-Agent", "")
    assert response_mock.last_request.headers.get("Content-Type") == "application/json"
    assert response_mock.last_request.headers.get("Custom-Header") == "value"

    assert result.status_code == 200


def test_api_node_preserves_custom_user_agent_header(requests_mock):
    """
    Tests that the API node preserves a custom User-Agent header if provided.
    """

    class TestAPINode(BaseAPINode):
        method = APIRequestMethod.GET
        url = "https://example.com/test"
        headers = {"User-Agent": "Custom-Agent/1.0"}

    response_mock = requests_mock.get(
        "https://example.com/test",
        json={"result": "success"},
        status_code=200,
    )

    node = TestAPINode()
    result = node.run()

    assert response_mock.last_request
    assert response_mock.last_request.headers.get("User-Agent") == "Custom-Agent/1.0"

    assert result.status_code == 200


def test_local_execute_api_with_hmac_secret(requests_mock):
    """Test that _local_execute_api adds HMAC headers when VELLUM_HMAC_SECRET is set."""

    class TestAPINode(BaseAPINode):
        method = APIRequestMethod.POST
        url = "https://example.com/test"
        json = {"test": "data"}

    response_mock = requests_mock.post(
        "https://example.com/test",
        json={"result": "success"},
        status_code=200,
    )

    with patch.dict(os.environ, {"VELLUM_HMAC_SECRET": "test-secret"}):
        node = TestAPINode()
        result = node.run()

    assert response_mock.last_request
    assert "X-Vellum-Timestamp" in response_mock.last_request.headers
    assert "X-Vellum-Signature" in response_mock.last_request.headers
    assert result.status_code == 200



# flake8: noqa: F403

# Force an import to ensure that all display classes are registered with the BaseNodeDisplay registry
from .base_node_display import BaseNodeDisplay
from .vellum import *
from .vellum import __all__ as all_vellum_display_nodes

__all__ = ["BaseNodeDisplay"] + all_vellum_display_nodes



from typing import Optional

from vellum.workflows.references import EnvironmentVariableReference


class EnvironmentVariables:
    @staticmethod
    def get(name: str, default: Optional[str] = None):
        env_ref = EnvironmentVariableReference(name=name)
        if default is not None:
            return env_ref.coalesce(default)
        return env_ref


# Deprecated: Use EnvironmentVariables instead. Will be removed in v0.15.0
Environment = EnvironmentVariables



from dataclasses import dataclass
from enum import Enum
import logging
from typing import Any, Dict

from vellum.client.types.vellum_error import VellumError
from vellum.client.types.vellum_error_code_enum import VellumErrorCodeEnum
from vellum.client.types.workflow_event_error import WorkflowEventError
from vellum.client.types.workflow_execution_event_error_code import WorkflowExecutionEventErrorCode

logger = logging.getLogger(__name__)


class WorkflowErrorCode(Enum):
    INVALID_WORKFLOW = "INVALID_WORKFLOW"
    INVALID_INPUTS = "INVALID_INPUTS"
    INVALID_OUTPUTS = "INVALID_OUTPUTS"
    INVALID_STATE = "INVALID_STATE"
    INVALID_CODE = "INVALID_CODE"
    INVALID_TEMPLATE = "INVALID_TEMPLATE"
    INTERNAL_ERROR = "INTERNAL_ERROR"
    NODE_EXECUTION = "NODE_EXECUTION"
    PROVIDER_CREDENTIALS_UNAVAILABLE = "PROVIDER_CREDENTIALS_UNAVAILABLE"
    PROVIDER_ERROR = "PROVIDER_ERROR"
    USER_DEFINED_ERROR = "USER_DEFINED_ERROR"
    WORKFLOW_CANCELLED = "WORKFLOW_CANCELLED"


@dataclass(frozen=True)
class WorkflowError:
    message: str
    code: WorkflowErrorCode

    def __contains__(self, item: Any) -> bool:
        return item in self.message


_VELLUM_ERROR_CODE_TO_WORKFLOW_ERROR_CODE: Dict[VellumErrorCodeEnum, WorkflowErrorCode] = {
    "INVALID_REQUEST": WorkflowErrorCode.INVALID_INPUTS,
    "INVALID_INPUTS": WorkflowErrorCode.INVALID_INPUTS,
    "PROVIDER_ERROR": WorkflowErrorCode.PROVIDER_ERROR,
    "REQUEST_TIMEOUT": WorkflowErrorCode.PROVIDER_ERROR,
    "INTERNAL_SERVER_ERROR": WorkflowErrorCode.INTERNAL_ERROR,
    "USER_DEFINED_ERROR": WorkflowErrorCode.USER_DEFINED_ERROR,
}


def vellum_error_to_workflow_error(error: VellumError) -> WorkflowError:
    workflow_error_code = _VELLUM_ERROR_CODE_TO_WORKFLOW_ERROR_CODE.get(error.code)

    if workflow_error_code is None:
        logger.warning(f"Unknown Vellum error code '{error.code}' encountered. Falling back to INTERNAL_ERROR.")
        workflow_error_code = WorkflowErrorCode.INTERNAL_ERROR

    return WorkflowError(
        message=error.message,
        code=workflow_error_code,
    )


_WORKFLOW_EVENT_ERROR_CODE_TO_WORKFLOW_ERROR_CODE: Dict[WorkflowExecutionEventErrorCode, WorkflowErrorCode] = {
    "WORKFLOW_INITIALIZATION": WorkflowErrorCode.INVALID_WORKFLOW,
    "WORKFLOW_CANCELLED": WorkflowErrorCode.WORKFLOW_CANCELLED,
    "NODE_EXECUTION_COUNT_LIMIT_REACHED": WorkflowErrorCode.INVALID_STATE,
    "INTERNAL_SERVER_ERROR": WorkflowErrorCode.INTERNAL_ERROR,
    "NODE_EXECUTION": WorkflowErrorCode.NODE_EXECUTION,
    "LLM_PROVIDER": WorkflowErrorCode.PROVIDER_ERROR,
    "INVALID_CODE": WorkflowErrorCode.INVALID_CODE,
    "INVALID_INPUTS": WorkflowErrorCode.INVALID_INPUTS,
    "INVALID_TEMPLATE": WorkflowErrorCode.INVALID_TEMPLATE,
    "USER_DEFINED_ERROR": WorkflowErrorCode.USER_DEFINED_ERROR,
}


def workflow_event_error_to_workflow_error(error: WorkflowEventError) -> WorkflowError:
    return WorkflowError(
        message=error.message,
        code=_WORKFLOW_EVENT_ERROR_CODE_TO_WORKFLOW_ERROR_CODE.get(error.code, WorkflowErrorCode.INTERNAL_ERROR),
    )


_WORKFLOW_ERROR_CODE_TO_VELLUM_ERROR_CODE: Dict[WorkflowErrorCode, VellumErrorCodeEnum] = {
    WorkflowErrorCode.INVALID_WORKFLOW: "INVALID_REQUEST",
    WorkflowErrorCode.INVALID_INPUTS: "INVALID_INPUTS",
    WorkflowErrorCode.INVALID_OUTPUTS: "INVALID_REQUEST",
    WorkflowErrorCode.INVALID_STATE: "INVALID_REQUEST",
    WorkflowErrorCode.INVALID_CODE: "INVALID_CODE",
    WorkflowErrorCode.INVALID_TEMPLATE: "INVALID_INPUTS",
    WorkflowErrorCode.INTERNAL_ERROR: "INTERNAL_SERVER_ERROR",
    WorkflowErrorCode.NODE_EXECUTION: "USER_DEFINED_ERROR",
    WorkflowErrorCode.PROVIDER_ERROR: "PROVIDER_ERROR",
    WorkflowErrorCode.USER_DEFINED_ERROR: "USER_DEFINED_ERROR",
    WorkflowErrorCode.WORKFLOW_CANCELLED: "REQUEST_TIMEOUT",
}


def workflow_error_to_vellum_error(error: WorkflowError) -> VellumError:
    return VellumError(
        message=error.message,
        code=_WORKFLOW_ERROR_CODE_TO_VELLUM_ERROR_CODE.get(error.code, "INTERNAL_SERVER_ERROR"),
    )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class BasicVectorizerIntfloatMultilingualE5Large(UniversalBaseModel):
    """
    Basic vectorizer for intfloat/multilingual-e5-large.
    """

    config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    model_name: typing.Literal["intfloat/multilingual-e5-large"] = "intfloat/multilingual-e5-large"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any, List, Optional, Union

from vellum.client.core.pydantic_utilities import UniversalBaseModel
from vellum.client.types.condition_combinator import ConditionCombinator
from vellum.client.types.logical_operator import LogicalOperator
from vellum.client.types.vellum_value_logical_condition_group_request import VellumValueLogicalConditionGroupRequest
from vellum.client.types.vellum_value_logical_condition_request import VellumValueLogicalConditionRequest
from vellum.workflows.nodes.displayable.bases.utils import primitive_to_vellum_value_request


class MetadataLogicalConditionGroup(UniversalBaseModel):
    combinator: ConditionCombinator
    negated: bool
    conditions: List["MetadataLogicalExpression"]

    def to_request(self) -> VellumValueLogicalConditionGroupRequest:
        return VellumValueLogicalConditionGroupRequest(
            combinator=self.combinator,
            negated=self.negated,
            conditions=[c.to_request() for c in self.conditions],
        )


class MetadataLogicalCondition(UniversalBaseModel):
    lhs_variable: Any
    operator: LogicalOperator
    rhs_variable: Any = None

    def to_request(self) -> VellumValueLogicalConditionRequest:
        return VellumValueLogicalConditionRequest(
            lhs_variable=primitive_to_vellum_value_request(self.lhs_variable),
            operator=self.operator,
            rhs_variable=primitive_to_vellum_value_request(self.rhs_variable),
        )


MetadataLogicalExpression = Union[MetadataLogicalConditionGroup, MetadataLogicalCondition]


class SearchFilters(UniversalBaseModel):
    external_ids: Optional[List[str]] = None
    metadata: Optional[MetadataLogicalConditionGroup] = None



import enum
import json
from typing import Any, List, Union, cast

from vellum.client.types.array_vellum_value import ArrayVellumValue
from vellum.client.types.array_vellum_value_request import ArrayVellumValueRequest
from vellum.client.types.audio_vellum_value import AudioVellumValue
from vellum.client.types.audio_vellum_value_request import AudioVellumValueRequest
from vellum.client.types.chat_history_vellum_value import ChatHistoryVellumValue
from vellum.client.types.chat_history_vellum_value_request import ChatHistoryVellumValueRequest
from vellum.client.types.chat_message import ChatMessage
from vellum.client.types.error_vellum_value import ErrorVellumValue
from vellum.client.types.error_vellum_value_request import ErrorVellumValueRequest
from vellum.client.types.function_call_vellum_value import FunctionCallVellumValue
from vellum.client.types.function_call_vellum_value_request import FunctionCallVellumValueRequest
from vellum.client.types.image_vellum_value import ImageVellumValue
from vellum.client.types.image_vellum_value_request import ImageVellumValueRequest
from vellum.client.types.json_vellum_value import JsonVellumValue
from vellum.client.types.json_vellum_value_request import JsonVellumValueRequest
from vellum.client.types.number_vellum_value import NumberVellumValue
from vellum.client.types.number_vellum_value_request import NumberVellumValueRequest
from vellum.client.types.search_result import SearchResult
from vellum.client.types.search_result_request import SearchResultRequest
from vellum.client.types.search_results_vellum_value import SearchResultsVellumValue
from vellum.client.types.search_results_vellum_value_request import SearchResultsVellumValueRequest
from vellum.client.types.string_vellum_value import StringVellumValue
from vellum.client.types.string_vellum_value_request import StringVellumValueRequest
from vellum.client.types.vellum_error import VellumError
from vellum.client.types.vellum_value import VellumValue
from vellum.client.types.vellum_value_request import VellumValueRequest
from vellum.client.types.video_vellum_value import VideoVellumValue
from vellum.client.types.video_vellum_value_request import VideoVellumValueRequest
from vellum.workflows.errors.types import WorkflowError, workflow_error_to_vellum_error
from vellum.workflows.state.encoder import DefaultStateEncoder

VELLUM_VALUE_REQUEST_TUPLE = (
    StringVellumValueRequest,
    NumberVellumValueRequest,
    JsonVellumValueRequest,
    AudioVellumValueRequest,
    VideoVellumValueRequest,
    ImageVellumValueRequest,
    FunctionCallVellumValueRequest,
    ErrorVellumValueRequest,
    ArrayVellumValueRequest,
    ChatHistoryVellumValueRequest,
    SearchResultsVellumValueRequest,
)


def primitive_to_vellum_value(value: Any) -> VellumValue:
    """Converts a python primitive to a VellumValue"""

    if isinstance(value, str):
        return StringVellumValue(value=value)
    elif isinstance(value, enum.Enum):
        return StringVellumValue(value=value.value)
    elif isinstance(value, bool):
        return JsonVellumValue(value=value)
    elif isinstance(value, (int, float)):
        return NumberVellumValue(value=value)
    elif isinstance(value, list) and (
        all(isinstance(message, ChatMessage) for message in value)
        or all(isinstance(message, ChatMessage) for message in value)
    ):
        chat_messages = cast(Union[List[ChatMessage], List[ChatMessage]], value)
        return ChatHistoryVellumValue(value=chat_messages)
    elif isinstance(value, list) and (
        all(isinstance(search_result, SearchResultRequest) for search_result in value)
        or all(isinstance(search_result, SearchResult) for search_result in value)
    ):
        search_results = cast(Union[List[SearchResultRequest], List[SearchResult]], value)
        return SearchResultsVellumValue(value=search_results)
    elif isinstance(value, VellumError):
        return ErrorVellumValue(value=value)
    elif isinstance(value, WorkflowError):
        return ErrorVellumValue(value=workflow_error_to_vellum_error(value))
    elif isinstance(
        value,
        (
            StringVellumValue,
            NumberVellumValue,
            JsonVellumValue,
            AudioVellumValue,
            VideoVellumValue,
            ImageVellumValue,
            FunctionCallVellumValue,
            ErrorVellumValue,
            ArrayVellumValue,
            ChatHistoryVellumValue,
            SearchResultsVellumValue,
        ),
    ):
        return value
    elif isinstance(
        value,
        VELLUM_VALUE_REQUEST_TUPLE,
    ):
        # This type ignore is safe because consumers of this function won't care the difference between
        # XVellumValue and XVellumValueRequest. Hopefully in the near future, neither will we
        return value  # type: ignore

    try:
        json_value = json.dumps(value, cls=DefaultStateEncoder)
    except json.JSONDecodeError:
        raise ValueError(f"Unsupported variable type: {value.__class__.__name__}")

    return JsonVellumValue(value=json.loads(json_value))


def primitive_to_vellum_value_request(value: Any) -> VellumValueRequest:
    vellum_value = primitive_to_vellum_value(value)
    vellum_value_request_class = next(
        (
            vellum_value_request_class
            for vellum_value_request_class in VELLUM_VALUE_REQUEST_TUPLE
            if vellum_value_request_class.__name__.startswith(vellum_value.__class__.__name__)
        ),
        None,
    )

    if vellum_value_request_class is None:
        raise ValueError(f"Unsupported variable type: {vellum_value.__class__.__name__}")

    return vellum_value_request_class.model_validate(vellum_value.model_dump())



import io
import json
import os
import zipfile
from typing import Optional, cast

import click
from dotenv import load_dotenv
from pydash import snake_case

from vellum.client.types.workflow_sandbox_example import WorkflowSandboxExample
from vellum.workflows.vellum_client import create_vellum_client
from vellum_cli.config import WorkflowConfig, load_vellum_cli_config
from vellum_cli.logger import load_cli_logger
from vellum_cli.pull import PullContentsMetadata, WorkflowConfigResolutionResult

ERROR_LOG_FILE_NAME = "error.log"
METADATA_FILE_NAME = "metadata.json"


def init_command(template_name: Optional[str] = None, target_directory: Optional[str] = None):
    load_dotenv()
    logger = load_cli_logger()
    config = load_vellum_cli_config()

    client = create_vellum_client()
    templates_response = client.workflow_sandboxes.list_workflow_sandbox_examples(tag="TEMPLATES")

    templates = templates_response.results
    if not templates:
        logger.error("No templates available")
        return

    if template_name:
        selected_template = next((t for t in templates if snake_case(t.label) == template_name), None)
        if not selected_template:
            logger.error(f"Template {template_name} not found")
            return
    else:
        click.echo(click.style("Available Templates", bold=True, fg="green"))
        for idx, template in enumerate(templates, 1):
            click.echo(f"{idx}. {template.label}")

        choice = click.prompt(
            f"Please select a template number (1-{len(templates)})", type=click.IntRange(1, len(templates))
        )
        selected_template = cast(WorkflowSandboxExample, templates[choice - 1])

        click.echo(click.style(f"\nYou selected: {selected_template.label}\n", bold=True, fg="cyan"))

    # Create workflow config with module name from template label
    workflow_config = WorkflowConfig(
        workflow_sandbox_id=selected_template.id,
        module=snake_case(selected_template.label),  # Set module name directly from template
    )
    config.workflows.append(workflow_config)

    workflow_config_result = WorkflowConfigResolutionResult(
        workflow_config=workflow_config,
        pk=selected_template.id,
    )

    pk = workflow_config_result.pk
    if not pk:
        raise ValueError("No workflow sandbox ID found in project to pull from.")

    # Use target_directory if provided, otherwise use current working directory
    base_dir = os.path.join(os.getcwd(), target_directory) if target_directory else os.getcwd()
    target_dir = os.path.join(base_dir, *workflow_config.module.split("."))
    workflow_config.target_directory = target_dir if target_directory else None

    if os.path.exists(target_dir):
        click.echo(click.style(f"{target_dir} already exists.", fg="red"))
        return

    logger.info(f"Pulling workflow into {workflow_config.module}...")

    query_parameters = {
        "include_sandbox": True,
    }

    response = client.workflows.pull(
        pk,
        request_options={"additional_query_parameters": query_parameters},
    )

    zip_bytes = b"".join(response)
    zip_buffer = io.BytesIO(zip_bytes)

    error_content = ""

    with zipfile.ZipFile(zip_buffer) as zip_file:
        if METADATA_FILE_NAME in zip_file.namelist():
            metadata_json: Optional[dict] = None
            with zip_file.open(METADATA_FILE_NAME) as source:
                metadata_json = json.load(source)

            pull_contents_metadata = PullContentsMetadata.model_validate(metadata_json)

            if pull_contents_metadata.runner_config:
                workflow_config.container_image_name = pull_contents_metadata.runner_config.container_image_name
                workflow_config.container_image_tag = pull_contents_metadata.runner_config.container_image_tag
                if workflow_config.container_image_name and not workflow_config.container_image_tag:
                    workflow_config.container_image_tag = "latest"

            if not workflow_config.module and pull_contents_metadata.label:
                workflow_config.module = snake_case(pull_contents_metadata.label)

        if not workflow_config.module:
            raise ValueError(f"Failed to resolve a module name for Workflow {pk}")

        for file_name in zip_file.namelist():
            with zip_file.open(file_name) as source:
                content = source.read().decode("utf-8")
                if file_name == ERROR_LOG_FILE_NAME:
                    error_content = content
                    continue
                if file_name == METADATA_FILE_NAME:
                    continue

                target_file = os.path.join(target_dir, file_name)
                os.makedirs(os.path.dirname(target_file), exist_ok=True)
                with open(target_file, "w") as target:
                    logger.info(f"Writing to {target_file}...")
                    target.write(content)

    config.save()

    if error_content:
        logger.error(error_content)
    else:
        logger.info(f"Successfully pulled Workflow into {target_dir}")



from collections.abc import Mapping
import dataclasses
from typing import Any, Sequence, Type, TypeVar, Union, get_args, get_origin

from pydantic import BaseModel, GetCoreSchemaHandler
from pydantic_core import core_schema

from vellum.workflows.descriptors.base import BaseDescriptor
from vellum.workflows.descriptors.exceptions import InvalidExpressionException
from vellum.workflows.descriptors.utils import is_resolved_instance, resolve_value
from vellum.workflows.state.base import BaseState

LHS = TypeVar("LHS")
AccessorField = Union[str, int, BaseDescriptor[str], BaseDescriptor[int]]


class AccessorExpression(BaseDescriptor[Any]):
    def __init__(
        self,
        *,
        base: BaseDescriptor[LHS],
        field: AccessorField,
    ) -> None:
        super().__init__(
            name=f"{base.name}.{field}",
            types=self._infer_accessor_types(base, field),
            instance=None,
        )
        self._base = base
        self._field = field

    def _infer_accessor_types(self, base: BaseDescriptor[LHS], field: AccessorField) -> tuple[Type, ...]:
        """
        Infer the types for this accessor expression based on the base descriptor's types
        and the field being accessed.
        """
        if not base.types:
            return ()

        inferred_types = []

        for base_type in base.types:
            origin = get_origin(base_type)
            args = get_args(base_type)

            if is_resolved_instance(field, int) and origin in (list, tuple) and args:
                if origin is list:
                    inferred_types.append(args[0])
                elif origin is tuple and len(args) == 2 and args[1] is ...:
                    inferred_types.append(args[0])
                elif origin is tuple and len(args) > abs(field):
                    if field >= 0:
                        inferred_types.append(args[field])
                    else:
                        inferred_types.append(args[field])
            elif is_resolved_instance(field, str) and origin in (dict,) and len(args) >= 2:
                inferred_types.append(args[1])  # Value type from Dict[K, V]

        return tuple(set(inferred_types)) if inferred_types else ()

    def resolve(self, state: "BaseState") -> Any:
        base = resolve_value(self._base, state)
        accessor_field = resolve_value(self._field, state)

        if dataclasses.is_dataclass(base):
            if isinstance(accessor_field, int):
                raise InvalidExpressionException("Cannot access field by index on a dataclass")

            try:
                return getattr(base, accessor_field)
            except AttributeError:
                raise InvalidExpressionException(
                    f"Field '{accessor_field}' not found on dataclass {type(base).__name__}"
                )

        if isinstance(base, BaseModel):
            if isinstance(accessor_field, int):
                raise InvalidExpressionException("Cannot access field by index on a BaseModel")

            try:
                return getattr(base, accessor_field)
            except AttributeError:
                raise InvalidExpressionException(
                    f"Field '{accessor_field}' not found on BaseModel {type(base).__name__}"
                )

        if isinstance(base, Mapping):
            try:
                return base[accessor_field]
            except KeyError:
                raise InvalidExpressionException(f"Key '{accessor_field}' not found in mapping")

        if isinstance(base, Sequence):
            try:
                index = int(accessor_field)
                return base[index]
            except (IndexError, ValueError):
                if isinstance(accessor_field, int) or (
                    isinstance(accessor_field, str) and accessor_field.lstrip("-").isdigit()
                ):
                    raise InvalidExpressionException(
                        f"Index {self._field} is out of bounds for sequence of length {len(base)}"
                    )
                else:
                    raise InvalidExpressionException(f"Invalid index '{self._field}' for sequence access")

        raise InvalidExpressionException(f"Cannot get field {self._field} from {base}")

    @classmethod
    def __get_pydantic_core_schema__(
        cls, source_type: Type[Any], handler: GetCoreSchemaHandler
    ) -> core_schema.CoreSchema:
        return core_schema.is_instance_schema(cls)



from typing import Generic, TypeVar, Union

from vellum.workflows.descriptors.base import BaseDescriptor
from vellum.workflows.descriptors.exceptions import InvalidExpressionException
from vellum.workflows.descriptors.utils import resolve_value
from vellum.workflows.state.base import BaseState

LHS = TypeVar("LHS")
RHS = TypeVar("RHS")


class ConcatExpression(BaseDescriptor[list], Generic[LHS, RHS]):
    def __init__(
        self,
        *,
        lhs: Union[BaseDescriptor[LHS], LHS],
        rhs: Union[BaseDescriptor[RHS], RHS],
    ) -> None:
        super().__init__(name=f"{lhs} + {rhs}", types=(list,))
        self._lhs = lhs
        self._rhs = rhs

    def resolve(self, state: "BaseState") -> list:
        lval = resolve_value(self._lhs, state)
        rval = resolve_value(self._rhs, state)

        if not isinstance(lval, list):
            raise InvalidExpressionException(f"Expected LHS to be a list, got {type(lval)}")
        if not isinstance(rval, list):
            raise InvalidExpressionException(f"Expected RHS to be a list, got {type(rval)}")

        return lval + rval
